@@ -1,28 +1,28 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2009 Edgewall Software
+# Copyright (C) 2005-2021 Edgewall Software
 # Copyright (C) 2005-2007 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import copy
 import os.path
 import re
-from ConfigParser import ConfigParser, ParsingError
-
-from genshi.builder import tag
+from configparser import (ConfigParser, NoOptionError, NoSectionError,
+                          ParsingError)
 
 from trac.admin import AdminCommandError, IAdminCommandProvider
 from trac.core import Component, ExtensionPoint, TracError, implements
 from trac.util import AtomicFile, as_bool
-from trac.util.compat import OrderedDict, wait_for_file_mtime_change
+from trac.util.compat import wait_for_file_mtime_change
+from trac.util.html import tag
 from trac.util.text import cleandoc, printout, to_unicode, to_utf8
 from trac.util.translation import N_, _, dgettext, tag_
 
@@ -45,7 +45,7 @@ def _getfloat(value):
 def _getlist(value, sep, keep_empty):
     if not value:
         return []
-    if isinstance(value, basestring):
+    if isinstance(value, str):
         if isinstance(sep, (list, tuple)):
             splitted = re.split('|'.join(map(re.escape, sep)), value)
         else:
@@ -73,8 +73,7 @@ class ConfigurationError(TracError):
     def __init__(self, message=None, title=None, show_traceback=False):
         if message is None:
             message = _("Look in the Trac log for more information.")
-        super(ConfigurationError, self).__init__(message, title,
-                                                 show_traceback)
+        super().__init__(message, title, show_traceback)
 
 
 class UnicodeConfigParser(ConfigParser):
@@ -89,55 +88,39 @@ class UnicodeConfigParser(ConfigParser):
     # The base class `RawConfigParser` doesn't inherit from `object`
     # so we can't use `super`.
 
-    def __init__(self, **kwargs):
-        dict_type = kwargs.pop('dict_type', None) or OrderedDict
-        ConfigParser.__init__(self, dict_type=dict_type, **kwargs)
+    def __init__(self, ignorecase_option=True, **kwargs):
+        self._ignorecase_option = ignorecase_option
+        kwargs.setdefault('interpolation', None)
+        ConfigParser.__init__(self, **kwargs)
 
-    def sections(self):
-        return map(to_unicode, ConfigParser.sections(self))
+    def optionxform(self, option):
+        if self._ignorecase_option:
+            option = option.lower()
+        return option
 
-    def add_section(self, section):
-        section_str = to_utf8(section)
-        ConfigParser.add_section(self, section_str)
+    def get(self, section, option, raw=False, vars=None,
+            fallback=_use_default):
+        try:
+            return ConfigParser.get(self, section, option, raw=raw, vars=vars)
+        except (NoSectionError, NoOptionError):
+            if fallback is _use_default:
+                raise
+            return fallback
 
-    def has_section(self, section):
-        section_str = to_utf8(section)
-        return ConfigParser.has_section(self, section_str)
-
-    def options(self, section):
-        section_str = to_utf8(section)
-        return map(to_unicode, ConfigParser.options(self, section_str))
-
-    def get(self, section, option, raw=False, vars=None):
-        section_str = to_utf8(section)
-        option_str = to_utf8(option)
-        return to_unicode(ConfigParser.get(self, section_str,
-                                           option_str, raw, vars))
-
-    def items(self, section, raw=False, vars=None):
-        section_str = to_utf8(section)
-        return [(to_unicode(k), to_unicode(v))
-                for k, v in ConfigParser.items(self, section_str, raw, vars)]
-
-    def has_option(self, section, option):
-        section_str = to_utf8(section)
-        option_str = to_utf8(option)
-        return ConfigParser.has_option(self, section_str, option_str)
+    def items(self, section, raw=False, vars=None, fallback=_use_default):
+        try:
+            return ConfigParser.items(self, section, raw=raw, vars=vars)
+        except NoSectionError:
+            if fallback is _use_default:
+                raise
+            return fallback
 
     def set(self, section, option, value=None):
-        section_str = to_utf8(section)
-        option_str = to_utf8(option)
-        value_str = to_utf8(value) if value is not None else ''
-        ConfigParser.set(self, section_str, option_str, value_str)
+        value_str = to_unicode(value if value is not None else '')
+        ConfigParser.set(self, section, option, value_str)
 
-    def remove_option(self, section, option):
-        section_str = to_utf8(section)
-        option_str = to_utf8(option)
-        ConfigParser.remove_option(self, section_str, option_str)
-
-    def remove_section(self, section):
-        section_str = to_utf8(section)
-        ConfigParser.remove_section(self, section_str)
+    def read(self, filename, encoding='utf-8'):
+        return ConfigParser.read(self, filename, encoding)
 
     def __copy__(self):
         parser = self.__class__()
@@ -181,6 +164,10 @@ class Configuration(object):
             self._sections[name] = Section(self, name)
         return self._sections[name]
 
+    def __delitem__(self, name):
+        self._sections.pop(name, None)
+        self.parser.remove_section(name)
+
     @property
     def exists(self):
         """Return boolean indicating configuration file existence.
@@ -264,7 +251,7 @@ class Configuration(object):
         """
         defaults = {}
         for (section, key), option in \
-                Option.get_registry(compmgr).iteritems():
+                Option.get_registry(compmgr).items():
             defaults.setdefault(section, {})[key] = \
                 option.dumps(option.default)
         return defaults
@@ -280,22 +267,30 @@ class Configuration(object):
         """
         return self[section].options(compmgr)
 
-    def remove(self, section, key):
-        """Remove the specified option."""
-        self[section].remove(key)
+    def remove(self, section, key=None):
+        """Remove the specified option or section."""
+        if key:
+            self[section].remove(key)
+        else:
+            del self[section]
 
-    def sections(self, compmgr=None, defaults=True):
+    def sections(self, compmgr=None, defaults=True, empty=False):
         """Return a list of section names.
 
         If `compmgr` is specified, only the section names corresponding to
         options declared in components that are enabled in the given
         `ComponentManager` are returned.
+
+        :param empty: If `True`, include sections from the registry that
+            contain no options.
         """
         sections = set(self.parser.sections())
         for parent in self.parents:
             sections.update(parent.sections(compmgr, defaults=False))
         if defaults:
             sections.update(self.defaults(compmgr))
+        if empty:
+            sections.update(ConfigSection.get_registry(compmgr))
         return sorted(sections)
 
     def has_option(self, section, option, defaults=True):
@@ -309,7 +304,7 @@ class Configuration(object):
         """Write the configuration options to the primary file."""
 
         all_options = {}
-        for (section, name), option in Option.get_registry().iteritems():
+        for (section, name), option in Option.get_registry().items():
             all_options.setdefault(section, {})[name] = option
 
         def normalize(section, name, value):
@@ -363,7 +358,7 @@ class Configuration(object):
                     raise TracError(_("Error reading '%(file)s', make sure "
                                       "it is readable.", file=self.filename))
             except ParsingError as e:
-                raise TracError(e)
+                raise TracError(e) from e
             self._lastmtime = modtime
             self._pristine_parser = copy.deepcopy(self.parser)
             changed = True
@@ -405,11 +400,11 @@ class Configuration(object):
                 clsname = (cls.__module__ + '.' + cls.__name__).lower() \
                                                                .split('.')
                 if clsname[:len(component)] == component:
-                    for option in cls.__dict__.itervalues():
+                    for option in cls.__dict__.values():
                         if isinstance(option, Option):
                             set_option_default(option)
         else:
-            for option in Option.get_registry(compmgr).itervalues():
+            for option in Option.get_registry(compmgr).values():
                 set_option_default(option)
 
     def _get_parents(self):
@@ -475,7 +470,7 @@ class Section(object):
                     options.add(loption)
                     yield option
         if defaults:
-            for section, option in Option.get_registry(compmgr).iterkeys():
+            for section, option in Option.get_registry(compmgr).keys():
                 if section == self.name and option.lower() not in options:
                     yield option
 
@@ -605,9 +600,11 @@ class Section(object):
         Like for `set()`, the changes won't persist until `save()` gets
         called.
         """
+        self._cache.pop(key, None)
         if self.config.parser.has_section(self.name):
-            self._cache.pop(key, None)
             self.config.parser.remove_option(self.name, key)
+            if not self.config.parser.options(self.name):
+                del self.config[self.name]
 
 
 def _get_registry(cls, compmgr=None):
@@ -622,11 +619,11 @@ def _get_registry(cls, compmgr=None):
     from trac.core import ComponentMeta
     components = {}
     for comp in ComponentMeta._components:
-        for attr in comp.__dict__.itervalues():
+        for attr in comp.__dict__.values():
             if isinstance(attr, cls):
                 components[attr] = comp
 
-    return dict(each for each in cls.registry.iteritems()
+    return dict(each for each in cls.registry.items()
                 if each[1] not in components
                    or compmgr.is_enabled(components[each[1]]))
 
@@ -692,11 +689,11 @@ class Option(object):
                  doc_domain='tracini', doc_args=None):
         """Create the configuration option.
 
-        @param section: the name of the configuration section this option
-            belongs to
-        @param name: the name of the option
-        @param default: the default value for the option
-        @param doc: documentation of the option
+        :param section: the name of the configuration section this option
+                        belongs to
+        :param name: the name of the option
+        :param default: the default value for the option
+        :param doc: documentation of the option
         """
         self.section = section
         self.name = name
@@ -735,7 +732,7 @@ class Option(object):
             return 'enabled'
         if value is False:
             return 'disabled'
-        if isinstance(value, unicode):
+        if isinstance(value, str):
             return value
         return to_unicode(value)
 
@@ -819,21 +816,28 @@ class ChoiceOption(Option):
     """
 
     def __init__(self, section, name, choices, doc='', doc_domain='tracini',
-                 doc_args=None):
+                 doc_args=None, case_sensitive=True):
         Option.__init__(self, section, name, to_unicode(choices[0]), doc,
                         doc_domain, doc_args)
-        self.choices = set(to_unicode(c).strip() for c in choices)
+        self.choices = list({to_unicode(c).strip() for c in choices})
+        self.case_sensitive = case_sensitive
 
     def accessor(self, section, name, default):
         value = section.get(name, default)
-        if value not in self.choices:
+        choices = self.choices[:]
+        if not self.case_sensitive:
+            choices = [c.lower() for c in choices]
+            value = value.lower()
+        try:
+            idx = choices.index(value)
+        except ValueError:
             raise ConfigurationError(
                     _('[%(section)s] %(entry)s: expected one of '
                       '(%(choices)s), got %(value)s',
                       section=section.name, entry=name, value=repr(value),
                       choices=', '.join('"%s"' % c
                                         for c in sorted(self.choices))))
-        return value
+        return self.choices[idx]
 
 
 class PathOption(Option):
@@ -914,15 +918,13 @@ class OrderedExtensionsOption(ListOption):
                          for idx, impl in enumerate(not_found)),
                      option=tag.code("[%s] %s" % (self.section, self.name))))
 
-        def compare(x, y):
-            x, y = x.__class__.__name__, y.__class__.__name__
-            if x not in order:
-                return int(y in order)
-            if y not in order:
-                return -int(x in order)
-            return cmp(order.index(x), order.index(y))
-        components.sort(compare)
-        return components
+        def key(impl):
+            name = impl.__class__.__name__
+            if name in order:
+                return 0, order.index(name)
+            else:
+                return 1, components.index(impl)
+        return sorted(components, key=key)
 
 
 class ConfigurationAdmin(Component):
@@ -936,8 +938,8 @@ class ConfigurationAdmin(Component):
         yield ('config get', '<section> <option>',
                'Get the value of the given option in "trac.ini"',
                self._complete_config, self._do_get)
-        yield ('config remove', '<section> <option>',
-               'Remove the specified option from "trac.ini"',
+        yield ('config remove', '<section> [<option>]',
+               'Remove the specified option or section from "trac.ini"',
                self._complete_config, self._do_remove)
         yield ('config set', '<section> <option> <value>',
                'Set the value for the given option in "trac.ini"',
@@ -945,7 +947,7 @@ class ConfigurationAdmin(Component):
 
     def _complete_config(self, args):
         if len(args) == 1:
-            return self.config.sections()
+            return self.config.sections(empty=True)
         elif len(args) == 2:
             return [name for (name, value) in self.config[args[0]].options()]
 
@@ -964,11 +966,14 @@ class ConfigurationAdmin(Component):
         if section == 'inherit' and option == 'file':
             self.config.parse_if_needed(force=True)  # Full reload
 
-    def _do_remove(self, section, option):
-        if not self.config.has_option(section, option):
+    def _do_remove(self, section, option=None):
+        if option and not self.config.has_option(section, option):
             raise AdminCommandError(
                 _("Option '%(option)s' doesn't exist in section"
                   " '%(section)s'", option=option, section=section))
+        elif section not in self.config:
+            raise AdminCommandError(
+                _("Section '%(section)s' doesn't exist", section=section))
         self.config.remove(section, option)
         self.config.save()
         if section == 'inherit' and option == 'file':
@@ -986,7 +991,7 @@ def get_configinfo(env):
     """
     all_options = {}
     for (section, name), option in \
-            Option.get_registry(env.compmgr).iteritems():
+            Option.get_registry(env.compmgr).items():
         all_options.setdefault(section, {})[name] = option
     sections = []
     for section in env.config.sections(env.compmgr):
@@ -997,8 +1002,8 @@ def get_configinfo(env):
                 default = registered.default
                 normalized = registered.normalize(value)
             else:
-                default = u''
-                normalized = unicode(value)
+                default = ''
+                normalized = str(value)
             options.append({'name': name, 'value': value,
                             'modified': normalized != default})
         options.sort(key=lambda o: o['name'])
