@@ -1,110 +1,38 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C)2005-2014 Edgewall Software
+# Copyright (C) 2005-2021 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
+import importlib
 import os
 import time
-import urllib
+import urllib.parse
 from abc import ABCMeta, abstractmethod
 
-from genshi.builder import tag
+from trac import db_default
+from trac.api import IEnvironmentSetupParticipant, ISystemInfoProvider
 from trac.config import BoolOption, ConfigurationError, IntOption, Option
 from trac.core import *
 from trac.db.pool import ConnectionPool
 from trac.db.schema import Table
 from trac.db.util import ConnectionWrapper
 from trac.util.concurrency import ThreadLocal
+from trac.util.html import tag
 from trac.util.text import unicode_passwd
 from trac.util.translation import _, tag_
 
 
-def with_transaction(env, db=None):
-    """Function decorator to emulate a context manager for database
-    transactions.
-
-    >>> def api_method(p1, p2):
-    >>>     result[0] = value1
-    >>>     @with_transaction(env)
-    >>>     def implementation(db):
-    >>>         # implementation
-    >>>         result[0] = value2
-    >>>     return result[0]
-
-    In this example, the `implementation()` function is called
-    automatically right after its definition, with a database
-    connection as an argument. If the function completes, a COMMIT is
-    issued on the connection. If the function raises an exception, a
-    ROLLBACK is issued and the exception is re-raised. Nested
-    transactions are supported, and a COMMIT will only be issued when
-    the outermost transaction block in a thread exits.
-
-    This mechanism is intended to replace the former practice of
-    getting a database connection with `env.get_db_cnx()` and issuing
-    an explicit commit or rollback, for mutating database
-    accesses. Its automatic handling of commit, rollback and nesting
-    makes it much more robust.
-
-    The optional `db` argument is intended for legacy code and should
-    not be used in new code.
-
-    :deprecated: This decorator is in turn deprecated in favor of
-                 context managers now that python 2.4 support has been
-                 dropped. It will be removed in Trac 1.3.1. Use instead
-                 the new context managers, `QueryContextManager` and
-                 `TransactionContextManager`, which make for much
-                 simpler to write code:
-
-    >>> def api_method(p1, p2):
-    >>>     result = value1
-    >>>     with env.db_transaction as db:
-    >>>         # implementation
-    >>>         result = value2
-    >>>     return result
-
-    """
-    dbm = DatabaseManager(env)
-    _transaction_local = dbm._transaction_local
-
-    def transaction_wrapper(fn):
-        ldb = _transaction_local.wdb
-        if db is not None:
-            if ldb is None:
-                _transaction_local.wdb = db
-                try:
-                    fn(db)
-                finally:
-                    _transaction_local.wdb = None
-            else:
-                assert ldb is db, "Invalid transaction nesting"
-                fn(db)
-        elif ldb:
-            fn(ldb)
-        else:
-            ldb = _transaction_local.wdb = dbm.get_connection()
-            try:
-                fn(ldb)
-                ldb.commit()
-                _transaction_local.wdb = None
-            except:
-                _transaction_local.wdb = None
-                ldb.rollback()
-                ldb = None
-                raise
-    return transaction_wrapper
-
-
 class DbContextManager(object):
     """Database Context Manager
 
@@ -138,9 +66,9 @@ class TransactionContextManager(DbContextManager):
     """
 
     def __enter__(self):
-        db = self.dbmgr._transaction_local.wdb # outermost writable db
+        db = self.dbmgr._transaction_local.wdb  # outermost writable db
         if not db:
-            db = self.dbmgr._transaction_local.rdb # reuse wrapped connection
+            db = self.dbmgr._transaction_local.rdb  # reuse wrapped connection
             if db:
                 db = ConnectionWrapper(db.cnx, db.log)
             else:
@@ -165,9 +93,9 @@ class QueryContextManager(DbContextManager):
     """
 
     def __enter__(self):
-        db = self.dbmgr._transaction_local.rdb # outermost readonly db
+        db = self.dbmgr._transaction_local.rdb  # outermost readonly db
         if not db:
-            db = self.dbmgr._transaction_local.wdb # reuse wrapped connection
+            db = self.dbmgr._transaction_local.wdb  # reuse wrapped connection
             if db:
                 db = ConnectionWrapper(db.cnx, db.log, readonly=True)
             else:
@@ -182,11 +110,9 @@ class QueryContextManager(DbContextManager):
                 self.db.close()
 
 
-class ConnectionBase(object):
+class ConnectionBase(object, metaclass=ABCMeta):
     """Abstract base class for database connection classes."""
 
-    __metaclass__ = ABCMeta
-
     @abstractmethod
     def cast(self, column, type):
         """Returns a clause casting `column` as `type`."""
@@ -219,11 +145,21 @@ class ConnectionBase(object):
         to `id`."""
         pass
 
+    @abstractmethod
+    def get_sequence_names(self):
+        """Returns a list of the sequence names."""
+        pass
+
     @abstractmethod
     def get_table_names(self):
         """Returns a list of the table names."""
         pass
 
+    @abstractmethod
+    def has_table(self, table):
+        """Returns whether the table exists."""
+        pass
+
     @abstractmethod
     def like(self):
         """Returns a case-insensitive `LIKE` clause."""
@@ -307,10 +243,18 @@ class IDatabaseConnector(Interface):
         """Backup the database to a location defined by
         trac.backup_dir"""
 
+    def get_system_info():
+        """Yield a sequence of `(name, version)` tuples describing the
+        name and version information of external packages used by the
+        connector.
+        """
+
 
 class DatabaseManager(Component):
     """Component used to manage the `IDatabaseConnector` implementations."""
 
+    implements(IEnvironmentSetupParticipant, ISystemInfoProvider)
+
     connectors = ExtensionPoint(IDatabaseConnector)
 
     connection_uri = Option('trac', 'database', 'sqlite:db/trac.db',
@@ -335,9 +279,14 @@ class DatabaseManager(Component):
 
     def init_db(self):
         connector, args = self.get_connector()
-        from trac.db_default import schema
-        args['schema'] = schema
+        args['schema'] = db_default.schema
         connector.init_db(**args)
+        version = db_default.db_version
+        self.set_database_version(version, 'initial_database_version')
+        self.set_database_version(version)
+
+    def insert_default_data(self):
+        self.insert_into_tables(db_default.get_data)
 
     def destroy_db(self):
         connector, args = self.get_connector()
@@ -367,9 +316,13 @@ class DatabaseManager(Component):
 
         :since: version 1.2
         """
+        table_name = table.name if isinstance(table, Table) else table
         with self.env.db_transaction as db:
+            if not db.has_table(table_name):
+                raise self.env.db_exc.OperationalError('Table %s not found' %
+                                                       db.quote(table_name))
             for col in columns:
-                db.drop_column(table, col)
+                db.drop_column(table_name, col)
 
     def drop_tables(self, schema):
         """Drop the specified tables.
@@ -387,14 +340,17 @@ class DatabaseManager(Component):
         """Insert data into existing tables.
 
         :param data_or_callable: Nested tuples of table names, column names
-                                 and row data:
-                                 (table1,
-                                  (column1, column2),
-                                  ((row1col1, row1col2), (row2col1, row2col2)),
-                                  table2, ...)
-                                or a callable that takes a single parameter
-                                `db` and returns the aforementioned nested
-                                tuple.
+                                 and row data::
+
+                                   (table1,
+                                    (column1, column2),
+                                    ((row1col1, row1col2),
+                                     (row2col1, row2col2)),
+                                    table2, ...)
+
+                                 or a callable that takes a single parameter
+                                 `db` and returns the aforementioned nested
+                                 tuple.
         :since: version 1.1.3
         """
         with self.env.db_transaction as db:
@@ -402,7 +358,7 @@ class DatabaseManager(Component):
                                         else data_or_callable
             for table, cols, vals in data:
                 db.executemany("INSERT INTO %s (%s) VALUES (%s)"
-                               % (table, ','.join(cols),
+                               % (db.quote(table), ','.join(cols),
                                   ','.join(['%s'] * len(cols))), vals)
 
     def reset_tables(self):
@@ -423,26 +379,34 @@ class DatabaseManager(Component):
 
         :since: version 1.2
         """
-        for new_table in new_schema:
-            temp_table_name = new_table.name + '_old'
-            old_column_names = set(self.get_column_names(new_table))
-            new_column_names = set(col.name for col in new_table.columns)
-            cols_to_copy = ','.join(old_column_names & new_column_names)
-            with self.env.db_transaction as db:
-                cursor = db.cursor()
-                cursor.execute("""
-                    CREATE TEMPORARY TABLE %s AS SELECT * FROM %s
-                    """ % (temp_table_name, new_table.name))
-                self.drop_tables((new_table,))
+        with self.env.db_transaction as db:
+            cursor = db.cursor()
+            for new_table in new_schema:
+                temp_table_name = new_table.name + '_old'
+                has_table = self.has_table(new_table)
+                if has_table:
+                    old_column_names = set(self.get_column_names(new_table))
+                    new_column_names = {col.name for col in new_table.columns}
+                    column_names = old_column_names & new_column_names
+                    if column_names:
+                        cols_to_copy = ','.join(db.quote(name)
+                                                for name in column_names)
+                        cursor.execute("""
+                            CREATE TEMPORARY TABLE %s AS SELECT * FROM %s
+                            """ % (db.quote(temp_table_name),
+                                   db.quote(new_table.name)))
+                    self.drop_tables((new_table,))
                 self.create_tables((new_table,))
-                cursor.execute("""
-                    INSERT INTO %s (%s) SELECT %s FROM %s
-                    """ % (new_table.name, cols_to_copy,
-                           cols_to_copy, temp_table_name))
-                for col in new_table.columns:
-                    if col.auto_increment:
-                        db.update_sequence(cursor, new_table.name, col.name)
-                self.drop_tables((temp_table_name,))
+                if has_table and column_names:
+                    cursor.execute("""
+                        INSERT INTO %s (%s) SELECT %s FROM %s
+                        """ % (db.quote(new_table.name), cols_to_copy,
+                               cols_to_copy, db.quote(temp_table_name)))
+                    for col in new_table.columns:
+                        if col.auto_increment:
+                            db.update_sequence(cursor, new_table.name,
+                                               col.name)
+                    self.drop_tables((temp_table_name,))
 
     def get_connection(self, readonly=False):
         """Get a database connection from the pool.
@@ -466,14 +430,25 @@ class DatabaseManager(Component):
                      in the SYSTEM table. Defaults to `database_version`,
                      which contains the database version for Trac.
         """
-        rows = self.env.db_query("""
-                SELECT value FROM system WHERE name=%s
-                """, (name,))
-        return int(rows[0][0]) if rows else False
+        with self.env.db_query as db:
+            for value, in db("""
+                    SELECT value FROM {0} WHERE name=%s
+                    """.format(db.quote('system')), (name,)):
+                return int(value)
+            else:
+                return False
 
     def get_exceptions(self):
         return self.get_connector()[0].get_exceptions()
 
+    def get_sequence_names(self):
+        """Returns a list of the sequence names.
+
+        :since: 1.3.2
+        """
+        with self.env.db_query as db:
+            return db.get_sequence_names()
+
     def get_table_names(self):
         """Returns a list of the table names.
 
@@ -485,14 +460,23 @@ class DatabaseManager(Component):
     def get_column_names(self, table):
         """Returns a list of the column names for `table`.
 
-        :param schema: a `Table` object or table name.
+        :param table: a `Table` object or table name.
 
         :since: 1.2
         """
         table_name = table.name if isinstance(table, Table) else table
         with self.env.db_query as db:
+            if not db.has_table(table_name):
+                raise self.env.db_exc.OperationalError('Table %s not found' %
+                                                       db.quote(table_name))
             return db.get_column_names(table_name)
 
+    def has_table(self, table):
+        """Returns whether the table exists."""
+        table_name = table.name if isinstance(table, Table) else table
+        with self.env.db_query as db:
+            return db.has_table(table_name)
+
     def set_database_version(self, version, name='database_version'):
         """Sets the database version in the SYSTEM table.
 
@@ -503,13 +487,15 @@ class DatabaseManager(Component):
         """
         current_database_version = self.get_database_version(name)
         if current_database_version is False:
-            self.env.db_transaction("""
-                    INSERT INTO system (name, value) VALUES (%s, %s)
-                    """, (name, version))
-        else:
-            self.env.db_transaction("""
-                    UPDATE system SET value=%s WHERE name=%s
-                    """, (version, name))
+            with self.env.db_transaction as db:
+                db("""
+                    INSERT INTO {0} (name, value) VALUES (%s, %s)
+                    """.format(db.quote('system')), (name, version))
+        elif version != self.get_database_version(name):
+            with self.env.db_transaction as db:
+                db("""
+                    UPDATE {0} SET value=%s WHERE name=%s
+                    """.format(db.quote('system')), (version, name))
             self.log.info("Upgraded %s from %d to %d",
                           name, current_database_version, version)
 
@@ -535,7 +521,7 @@ class DatabaseManager(Component):
                       name, dbver, version)
         return True
 
-    def upgrade(self, version, name='database_version', pkg=None):
+    def upgrade(self, version, name='database_version', pkg='trac.upgrades'):
         """Invokes `do_upgrade(env, version, cursor)` in module
         `"%s/db%i.py" % (pkg, version)`, for each required version upgrade.
 
@@ -549,19 +535,15 @@ class DatabaseManager(Component):
         """
         dbver = self.get_database_version(name)
         for i in range(dbver + 1, version + 1):
-            module = 'db%i' % i
+            module = '%s.db%i' % (pkg, i)
             try:
-                upgrades = __import__(pkg, globals(), locals(), [module])
+                upgrader = importlib.import_module(module)
             except ImportError:
-                raise TracError(_("No upgrade package %(pkg)s", pkg=pkg))
-            try:
-                script = getattr(upgrades, module)
-            except AttributeError:
                 raise TracError(_("No upgrade module %(module)s.py",
                                   module=module))
             with self.env.db_transaction as db:
                 cursor = db.cursor()
-                script.do_upgrade(self.env, i, cursor)
+                upgrader.do_upgrade(self.env, i, cursor)
                 self.set_database_version(i, name)
 
     def shutdown(self, tid=None):
@@ -623,12 +605,28 @@ class DatabaseManager(Component):
             args['log'] = self.log
         return connector, args
 
-    _get_connector = get_connector  # For 0.11 compatibility
+    # IEnvironmentSetupParticipant methods
+
+    def environment_created(self):
+        pass
+
+    def environment_needs_upgrade(self):
+        return self.needs_upgrade(db_default.db_version)
+
+    def upgrade_environment(self):
+        self.upgrade(db_default.db_version)
+
+    # ISystemInfoProvider methods
+
+    def get_system_info(self):
+        connector = self.get_connector()[0]
+        for info in connector.get_system_info():
+            yield info
 
 
 def get_column_names(cursor):
     """Retrieve column names from a cursor, if possible."""
-    return [unicode(d[0], 'utf-8') if isinstance(d[0], str) else d[0]
+    return [str(d[0], 'utf-8') if isinstance(d[0], bytes) else d[0]
             for d in cursor.description] if cursor.description else []
 
 
@@ -646,7 +644,7 @@ def parse_connection_uri(db_str):
         section = tag.a("[trac]",
                         title=_("TracIni documentation"),
                         class_='trac-target-new',
-                        href='http://trac.edgewall.org/wiki/TracIni'
+                        href='https://trac.edgewall.org/wiki/TracIni'
                              '#trac-section')
         raise ConfigurationError(
             tag_("Database connection string is empty. Set the %(option)s "
@@ -690,9 +688,9 @@ def parse_connection_uri(db_str):
         else:
             password = None
         if user:
-            user = urllib.unquote(user)
+            user = urllib.parse.unquote(user)
         if password:
-            password = unicode_passwd(urllib.unquote(password))
+            password = unicode_passwd(urllib.parse.unquote(password))
     else:
         user = password = None
 
@@ -721,16 +719,12 @@ def parse_connection_uri(db_str):
                 name, value = param.split('=', 1)
             except ValueError:
                 raise _invalid_db_str(db_str)
-            value = urllib.unquote(value)
+            value = urllib.parse.unquote(value)
             params[name] = value
 
     args = zip(('user', 'password', 'host', 'port', 'path', 'params'),
                (user, password, host, port, path, params))
-    return scheme, dict([(key, value) for key, value in args if value])
-
-
-# Compatibility for Trac < 1.1.3. Will be removed in 1.3.1.
-_parse_db_str = parse_connection_uri
+    return scheme, {key: value for key, value in args if value}
 
 
 def _invalid_db_str(db_str):
@@ -744,5 +738,5 @@ def _doc_db_str():
     return tag.a(_("documentation"),
                  title=_("Database Connection Strings documentation"),
                  class_='trac-target-new',
-                 href='http://trac.edgewall.org/wiki/'
+                 href='https://trac.edgewall.org/wiki/'
                       'TracIni#DatabaseConnectionStrings')
