@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2009 Edgewall Software
+# Copyright (C) 2005-2021 Edgewall Software
 # Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005 Jeff Weiss <trac@jeffweiss.org>
 # Copyright (C) 2006 Andres Salomon <dilinger@athenacr.com>
@@ -8,26 +8,26 @@
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import os
 import re
 import sys
-import types
-
-from genshi.core import Markup
+from contextlib import closing
+from subprocess import Popen, PIPE
 
+from trac.api import IEnvironmentSetupParticipant
 from trac.core import *
 from trac.config import Option
 from trac.db.api import ConnectionBase, DatabaseManager, IDatabaseConnector, \
                         get_column_names, parse_connection_uri
 from trac.db.util import ConnectionWrapper, IterableCursor
-from trac.env import IEnvironmentSetupParticipant, ISystemInfoProvider
 from trac.util import as_int, get_pkginfo
+from trac.util.html import Markup
 from trac.util.compat import close_fds
 from trac.util.text import exception_to_unicode, to_unicode
 from trac.util.translation import _
@@ -35,33 +35,34 @@ from trac.util.translation import _
 _like_escape_re = re.compile(r'([/_%])')
 
 try:
-    import MySQLdb
-    import MySQLdb.cursors
+    import pymysql
 except ImportError:
-    has_mysqldb = False
-    mysqldb_version = None
+    pymysql = None
+    pymsql_version = None
 else:
-    has_mysqldb = True
-    mysqldb_version = get_pkginfo(MySQLdb).get('version', MySQLdb.__version__)
+    pymsql_version = get_pkginfo(pymysql).get('version', pymysql.__version__)
 
-    class MySQLUnicodeCursor(MySQLdb.cursors.Cursor):
-        def _convert_row(self, row):
-            return tuple(v.decode('utf-8') if isinstance(v, str) else v
-                         for v in row)
+    class MySQLUnicodeCursor(pymysql.cursors.Cursor):
+        def execute(self, query, args=None):
+            if args:
+                args = tuple(str(arg) if isinstance(arg, Markup) else arg
+                             for arg in args)
+            return super().execute(query, args)
 
-        def fetchone(self):
-            row = super(MySQLUnicodeCursor, self).fetchone()
-            return self._convert_row(row) if row else None
-
-        def fetchmany(self, num):
-            rows = super(MySQLUnicodeCursor, self).fetchmany(num)
-            return [self._convert_row(row) for row in rows] \
-                   if rows is not None else []
+        def executemany(self, query, args):
+            if args:
+                args = [tuple(str(item) if isinstance(item, Markup) else item
+                              for item in arg)
+                        for arg in args]
+            return super().executemany(query, args)
 
         def fetchall(self):
-            rows = super(MySQLUnicodeCursor, self).fetchall()
-            return [self._convert_row(row) for row in rows] \
-                   if rows is not None else []
+            return list(super().fetchall())
+
+    class MySQLSilentCursor(MySQLUnicodeCursor):
+        def _show_warnings(self, conn=None):
+            pass
+
 
 # Mapping from "abstract" SQL types to DB-specific types
 _type_map = {
@@ -70,13 +71,19 @@ _type_map = {
 }
 
 
+def _quote(identifier):
+    return "`%s`" % identifier.replace('`', '``')
+
+
 class MySQLConnector(Component):
     """Database connector for MySQL version 4.1 and greater.
 
-    Database URLs should be of the form:
-    {{{
-    mysql://user[:password]@host[:port]/database[?param1=value&param2=value]
-    }}}
+    Database URLs should be of the form::
+
+      {{{
+      mysql://user[:password]@host[:port]/database[?param1=value&param2=value]
+      }}}
+
     The following parameters are supported:
      * `compress`: Enable compression (0 or 1)
      * `init_command`: Command to run once the connection is created
@@ -85,8 +92,7 @@ class MySQLConnector(Component):
      * `read_default_group`: Configuration group to use from the default file
      * `unix_socket`: Use a Unix socket at the given path to connect
     """
-    implements(IDatabaseConnector, IEnvironmentSetupParticipant,
-               ISystemInfoProvider)
+    implements(IDatabaseConnector, IEnvironmentSetupParticipant)
 
     required = False
 
@@ -94,22 +100,17 @@ class MySQLConnector(Component):
         """Location of mysqldump for MySQL database backups""")
 
     def __init__(self):
-        self._mysql_version = None
-        self.error = None
-
-    # ISystemInfoProvider methods
-
-    def get_system_info(self):
-        if self.required:
-            yield 'MySQL', self._mysql_version
-            yield 'MySQLdb', mysqldb_version
+        if pymysql:
+            self._mysql_version = \
+                'server: (not-connected), client: "%s", thread-safe: %s' % \
+                (pymysql.get_client_info(), pymysql.thread_safe())
+        else:
+            self._mysql_version = None
 
     # IDatabaseConnector methods
 
     def get_supported_schemes(self):
-        if not has_mysqldb:
-            self.error = _("Cannot load Python bindings for MySQL")
-        yield ('mysql', -1 if self.error else 1)
+        yield 'mysql', 1
 
     def get_connection(self, path, log=None, user=None, password=None,
                        host=None, port=None, params={}):
@@ -117,25 +118,25 @@ class MySQLConnector(Component):
         if not self.required:
             self._mysql_version = \
                 'server: "%s", client: "%s", thread-safe: %s' \
-                % (cnx.cnx.get_server_info(), MySQLdb.get_client_info(),
-                   MySQLdb.thread_safe())
+                % (cnx.cnx.get_server_info(), pymysql.get_client_info(),
+                   pymysql.thread_safe())
             self.required = True
         return cnx
 
     def get_exceptions(self):
-        return MySQLdb
+        return pymysql
 
     def init_db(self, path, schema=None, log=None, user=None, password=None,
                 host=None, port=None, params={}):
         cnx = self.get_connection(path, log, user, password, host, port,
                                   params)
         self._verify_variables(cnx)
-        utf8_size = self._utf8_size(cnx)
+        max_bytes = self._max_bytes(cnx)
         cursor = cnx.cursor()
         if schema is None:
             from trac.db_default import schema
         for table in schema:
-            for stmt in self.to_sql(table, utf8_size=utf8_size):
+            for stmt in self.to_sql(table, max_bytes=max_bytes):
                 self.log.debug(stmt)
                 cursor.execute(stmt)
         self._verify_table_status(cnx)
@@ -155,29 +156,31 @@ class MySQLConnector(Component):
                                   params)
         return bool(cnx.get_table_names())
 
-    def _utf8_size(self, cnx):
+    def _max_bytes(self, cnx):
         if cnx is None:
             connector, args = DatabaseManager(self.env).get_connector()
-            cnx = connector.get_connection(**args)
-            charset = cnx.charset
-            cnx.close()
+            with closing(connector.get_connection(**args)) as cnx:
+                charset = cnx.charset
         else:
             charset = cnx.charset
         return 4 if charset == 'utf8mb4' else 3
 
-    def _collist(self, table, columns, utf8_size):
+    _max_key_length = 3072
+
+    def _collist(self, table, columns, max_bytes):
         """Take a list of columns and impose limits on each so that indexing
         works properly.
 
-        Some Versions of MySQL limit each index prefix to 1000 bytes total,
+        Some Versions of MySQL limit each index prefix to 3072 bytes total,
         with a max of 767 bytes per column.
         """
         cols = []
-        limit_col = 767 / utf8_size
-        limit = min(1000 / (utf8_size * len(columns)), limit_col)
+        limit_col = 767 // max_bytes
+        limit = min(self._max_key_length // (max_bytes * len(columns)),
+                    limit_col)
         for c in columns:
-            name = '`%s`' % c
-            table_col = filter((lambda x: x.name == c), table.columns)
+            name = _quote(c)
+            table_col = list(filter((lambda x: x.name == c), table.columns))
             if len(table_col) == 1 and table_col[0].type.lower() == 'text':
                 if table_col[0].key_size is not None:
                     name += '(%d)' % min(table_col[0].key_size, limit_col)
@@ -188,10 +191,10 @@ class MySQLConnector(Component):
             cols.append(name)
         return ','.join(cols)
 
-    def to_sql(self, table, utf8_size=None):
-        if utf8_size is None:
-            utf8_size = self._utf8_size(None)
-        sql = ['CREATE TABLE %s (' % table.name]
+    def to_sql(self, table, max_bytes=None):
+        if max_bytes is None:
+            max_bytes = self._max_bytes(None)
+        sql = ['CREATE TABLE %s (' % _quote(table.name)]
         coldefs = []
         for column in table.columns:
             ctype = column.type
@@ -201,19 +204,20 @@ class MySQLConnector(Component):
                 # Override the column type, as a text field cannot
                 # use auto_increment.
                 column.type = 'int'
-            coldefs.append('    `%s` %s' % (column.name, ctype))
+            coldefs.append('    %s %s' % (_quote(column.name), ctype))
         if len(table.key) > 0:
             coldefs.append('    PRIMARY KEY (%s)' %
                            self._collist(table, table.key,
-                                         utf8_size=utf8_size))
+                                         max_bytes=max_bytes))
         sql.append(',\n'.join(coldefs) + '\n)')
         yield '\n'.join(sql)
 
         for index in table.indices:
             unique = 'UNIQUE' if index.unique else ''
-            yield 'CREATE %s INDEX %s_%s_idx ON %s (%s);' % (unique, table.name,
-                  '_'.join(index.columns), table.name,
-                  self._collist(table, index.columns, utf8_size=utf8_size))
+            idxname = '%s_%s_idx' % (table.name, '_'.join(index.columns))
+            yield 'CREATE %s INDEX %s ON %s (%s)' % \
+                  (unique, _quote(idxname), _quote(table.name),
+                   self._collist(table, index.columns, max_bytes=max_bytes))
 
     def alter_column_types(self, table, columns):
         """Yield SQL statements altering the type of one or more columns of
@@ -223,7 +227,7 @@ class MySQLConnector(Component):
         to `(from, to)` SQL type tuples.
         """
         alterations = []
-        for name, (from_, to) in sorted(columns.iteritems()):
+        for name, (from_, to) in sorted(columns.items()):
             to = _type_map.get(to, to)
             if to != _type_map.get(from_, from_):
                 alterations.append((name, to))
@@ -233,20 +237,19 @@ class MySQLConnector(Component):
                           for each in alterations))
 
     def backup(self, dest_file):
-        from subprocess import Popen, PIPE
         db_url = self.env.config.get('trac', 'database')
         scheme, db_prop = parse_connection_uri(db_url)
         db_params = db_prop.setdefault('params', {})
         db_name = os.path.basename(db_prop['path'])
 
-        args = [self.mysqldump_path]
+        args = [self.mysqldump_path, '--no-defaults']
         if 'host' in db_prop:
             args.extend(['-h', db_prop['host']])
         if 'port' in db_prop:
             args.extend(['-P', str(db_prop['port'])])
         if 'user' in db_prop:
             args.extend(['-u', db_prop['user']])
-        for name, value in db_params.iteritems():
+        for name, value in db_params.items():
             if name == 'compress' and as_int(value, 0):
                 args.append('--compress')
             elif name == 'named_pipe' and as_int(value, 0):
@@ -277,6 +280,10 @@ class MySQLConnector(Component):
             raise TracError(_("No destination file created"))
         return dest_file
 
+    def get_system_info(self):
+        yield 'MySQL', self._mysql_version
+        yield pymysql.__name__, pymsql_version
+
     # IEnvironmentSetupParticipant methods
 
     def environment_created(self):
@@ -330,7 +337,7 @@ class MySQLConnector(Component):
                        "'default_storage_engine','storage_engine',"
                        "'default_tmp_storage_engine',"
                        "'character_set_database','collation_database')")
-        vars = dict((row[0].lower(), row[1]) for row in cursor)
+        vars = {row[0].lower(): row[1] for row in cursor}
 
         engine = vars.get('default_storage_engine') or \
                  vars.get('storage_engine')
@@ -369,33 +376,39 @@ class MySQLConnection(ConnectionBase, ConnectionWrapper):
             password = ''
         if port is None:
             port = 3306
-        opts = {}
-        for name, value in params.iteritems():
-            key = name.encode('utf-8')
+        opts = {'charset': 'utf8'}
+        for name, value in params.items():
             if name == 'read_default_group':
-                opts[key] = value
+                opts[name] = value
             elif name == 'init_command':
-                opts[key] = value.encode('utf-8')
+                opts[name] = value
             elif name in ('read_default_file', 'unix_socket'):
-                opts[key] = value.encode(sys.getfilesystemencoding())
+                opts[name] = value
             elif name in ('compress', 'named_pipe'):
-                opts[key] = as_int(value, 0)
+                opts[name] = as_int(value, 0)
+            elif name == 'charset':
+                value = value.lower()
+                if value in ('utf8', 'utf8mb4'):
+                    opts[name] = value
+                else:
+                    self.log.warning("Invalid connection string parameter "
+                                     "'%s=%s'", name, value)
             else:
                 self.log.warning("Invalid connection string parameter '%s'",
                                  name)
-        cnx = MySQLdb.connect(db=path, user=user, passwd=password, host=host,
-                              port=port, charset='utf8', **opts)
-        self.schema = path
-        if hasattr(cnx, 'encoders'):
-            # 'encoders' undocumented but present since 1.2.1 (r422)
-            cnx.encoders[Markup] = cnx.encoders[types.UnicodeType]
+        cnx = pymysql.connect(db=path, user=user, passwd=password, host=host,
+                              port=port, **opts)
         cursor = cnx.cursor()
         cursor.execute("SHOW VARIABLES WHERE "
                        " variable_name='character_set_database'")
         self.charset = cursor.fetchone()[1]
-        if self.charset != 'utf8':
-            cnx.query("SET NAMES %s" % self.charset)
-            cnx.store_result()
+        cursor.close()
+        if self.charset != opts['charset']:
+            cnx.close()
+            opts['charset'] = self.charset
+            cnx = pymysql.connect(db=path, user=user, passwd=password,
+                                  host=host, port=port, **opts)
+        self.schema = path
         ConnectionWrapper.__init__(self, cnx, log)
         self._is_closed = False
 
@@ -406,14 +419,14 @@ class MySQLConnection(ConnectionBase, ConnectionWrapper):
         self.cnx.ping()
         try:
             self.cnx.rollback()
-        except MySQLdb.ProgrammingError:
+        except pymysql.ProgrammingError:
             self._is_closed = True
 
     def close(self):
         if not self._is_closed:
             try:
                 self.cnx.close()
-            except MySQLdb.ProgrammingError:
+            except pymysql.ProgrammingError:
                 pass # this error would mean it's already closed.  So, ignore
             self._is_closed = True
 
@@ -428,7 +441,7 @@ class MySQLConnection(ConnectionBase, ConnectionWrapper):
         return 'concat(%s)' % ', '.join(args)
 
     def drop_column(self, table, column):
-        cursor = MySQLdb.cursors.Cursor(self.cnx)
+        cursor = pymysql.cursors.Cursor(self.cnx)
         if column in self.get_column_names(table):
             quoted_table = self.quote(table)
             cursor.execute("SHOW INDEX FROM %s" % quoted_table)
@@ -438,7 +451,7 @@ class MySQLConnection(ConnectionBase, ConnectionWrapper):
                 row = dict(zip(columns, row))
                 keys.setdefault(row['Key_name'], []).append(row['Column_name'])
             # drop all composite indices which in the given column is involved
-            for key, columns in keys.iteritems():
+            for key, columns in keys.items():
                 if len(columns) > 1 and column in columns:
                     if key == 'PRIMARY':
                         cursor.execute("ALTER TABLE %s DROP PRIMARY KEY" %
@@ -450,8 +463,7 @@ class MySQLConnection(ConnectionBase, ConnectionWrapper):
                            (quoted_table, self.quote(column)))
 
     def drop_table(self, table):
-        cursor = MySQLdb.cursors.Cursor(self.cnx)
-        cursor._defer_warnings = True  # ignore "Warning: Unknown table ..."
+        cursor = MySQLSilentCursor(self.cnx)
         cursor.execute("DROP TABLE IF EXISTS " + self.quote(table))
 
     def get_column_names(self, table):
@@ -465,6 +477,9 @@ class MySQLConnection(ConnectionBase, ConnectionWrapper):
     def get_last_id(self, cursor, table, column='id'):
         return cursor.lastrowid
 
+    def get_sequence_names(self):
+        return []
+
     def get_table_names(self):
         rows = self.execute("""
             SELECT table_name FROM information_schema.tables
@@ -472,6 +487,13 @@ class MySQLConnection(ConnectionBase, ConnectionWrapper):
             """, (self.schema,))
         return [row[0] for row in rows]
 
+    def has_table(self, table):
+        rows = self.execute("""
+            SELECT EXISTS (SELECT * FROM information_schema.columns
+                           WHERE table_schema=%s AND table_name=%s)
+            """, (self.schema, table))
+        return bool(rows[0][0])
+
     def like(self):
         return "LIKE %%s COLLATE %s_general_ci ESCAPE '/'" % self.charset
 
@@ -484,19 +506,25 @@ class MySQLConnection(ConnectionBase, ConnectionWrapper):
             return table_names
         cursor = self.cursor()
         cursor.execute("""
-            SELECT table_name, auto_increment
-            FROM information_schema.tables
-            WHERE table_schema=%s""", (self.schema,))
-        for table, auto_increment in cursor.fetchall():
+            SELECT t.table_name,
+                   EXISTS (SELECT * FROM information_schema.columns AS c
+                           WHERE c.table_schema=t.table_schema
+                           AND c.table_name=t.table_name
+                           AND extra='auto_increment')
+            FROM information_schema.tables AS t
+            WHERE t.table_schema=%s
+            """, (self.schema,))
+        for table, has_autoinc in cursor.fetchall():
             table_names.append(table)
-            if auto_increment is None or auto_increment == 1:
+            quoted = self.quote(table)
+            if not has_autoinc:
                 # DELETE FROM is preferred to TRUNCATE TABLE, as the
-                # auto_increment is not used or it is 1.
-                cursor.execute("DELETE FROM %s" % table)
+                # auto_increment is not used.
+                cursor.execute("DELETE FROM %s" % quoted)
             else:
                 # TRUNCATE TABLE is preferred to DELETE FROM, as we
                 # need to reset the auto_increment in MySQL.
-                cursor.execute("TRUNCATE TABLE %s" % table)
+                cursor.execute("TRUNCATE TABLE %s" % quoted)
         return table_names
 
     def prefix_match(self):
@@ -506,7 +534,8 @@ class MySQLConnection(ConnectionBase, ConnectionWrapper):
         return self.like_escape(prefix) + '%'
 
     def quote(self, identifier):
-        return "`%s`" % identifier.replace('`', '``')
+        """Return the quoted identifier."""
+        return _quote(identifier)
 
     def update_sequence(self, cursor, table, column='id'):
         # MySQL handles sequence updates automagically
