@@ -1,36 +1,90 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2011 Edgewall Software
+# Copyright (C) 2011-2021 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 """Extra commands for setup.py.
 
-In addition to providing a few extra command classes in `l10n_cmdclass`,
-we also modify the standard `distutils.command.build` and
-`setuptools.command.install_lib` classes so that the relevant l10n commands
-for compiling catalogs are issued upon install.
+We provide a few extra command classes in `l10n_cmdclass` for
+localization tasks.  We also modify the standard commands
+`distutils.command.build` and `setuptools.command.install_lib` classes
+in order to call the l10n commands for compiling catalogs at the right
+time during install.
+
 """
 
-from StringIO import StringIO
-from itertools import izip
+from html.parser import HTMLParser
+import io
 import os
 import re
 from tokenize import generate_tokens, COMMENT, NAME, OP, STRING
 
-from distutils import log
+from jinja2.ext import babel_extract as jinja2_extractor
+
+from distutils import log as distlog
 from distutils.cmd import Command
 from distutils.command.build import build as _build
 from distutils.errors import DistutilsOptionError
 from setuptools.command.install_lib import install_lib as _install_lib
 
+
+def simplify_message(message):
+    """Transforms an extracted messsage (string or tuple) into one in
+    which the repeated white-space has been simplified to a single
+    space.
+
+    """
+    tuple_len = len(message) if isinstance(message, tuple) else 0
+    if tuple_len:
+        message = message[0]
+    message = ' '.join(message.split())
+    if tuple_len:
+        message = (message,) + (None,) * (tuple_len - 1)
+    return message
+
+
+class ScriptExtractor(HTMLParser):
+    def __init__(self, out):
+        HTMLParser.__init__(self)
+        self.out = out
+        self.in_javascript = False
+
+    def handle_starttag(self, tag, attrs):
+        if tag == 'script':
+            self.in_javascript = True
+
+    def handle_startendtag(self, tag, attrs):
+        self.in_javascript = False
+
+    def handle_charref(self, name):
+        if self.in_javascript:
+            self.out.write('&#%s;' % name)
+
+    def handle_entityref(self, name):
+        if self.in_javascript:
+            self.out.write('&%s;' % name)
+
+    def handle_data(self, data):
+        if self.in_javascript:
+            self.out.write(data)
+
+    def handle_endtag(self, tag):
+        self.in_javascript = False
+
+    def no_op(*args, **kwargs):
+        pass
+
+    handle_comment = handle_decl = handle_pi = no_op
+
+
 try:
     from babel.messages.catalog import TranslationError
     from babel.messages.extract import extract_javascript
@@ -40,9 +94,6 @@ try:
     from babel.support import Translations
     from babel.util import parse_encoding
 
-    _GENSHI_MARKUP_SEARCH = re.compile(r'\[[0-9]+:').search
-
-
     _DEFAULT_KWARGS_MAPS = {
         'Option': {'doc': 4},
         'BoolOption': {'doc': 4},
@@ -141,7 +192,7 @@ try:
                     else:
                         messages.append(None)
 
-                    for name, message in messages_kwargs.iteritems():
+                    for name, message in messages_kwargs.items():
                         if name not in func_kwargs_map:
                             continue
                         index = func_kwargs_map[name]
@@ -178,7 +229,7 @@ try:
                     # aid=617979&group_id=5470
                     value = eval('# coding=%s\n%s' % (encoding, value),
                                  {'__builtins__':{}}, {})
-                    if isinstance(value, str):
+                    if isinstance(value, bytes):
                         value = value.decode(encoding)
                     buf.append(value)
                 elif tok == OP and value == '=' and prev_tok == NAME:
@@ -212,22 +263,48 @@ try:
 
 
     def extract_javascript_script(fileobj, keywords, comment_tags, options):
-        """Extract messages from Javascript embedding in <script> tags.
+        """Extract messages from Javascript embedded in <script> tags.
 
         Select <script type="javascript/text"> tags and delegate to
         `extract_javascript`.
         """
-        from genshi.core import Stream
-        from genshi.input import XMLParser
-
-        out = StringIO()
-        stream = Stream(XMLParser(fileobj))
-        stream = stream.select('//script[@type="text/javascript"]')
-        stream.render(out=out, encoding='utf-8')
+        if not fileobj.name:
+            return []
+        out = io.StringIO()
+        extractor = ScriptExtractor(out)
+        extractor.feed(str(fileobj.read(), 'utf-8'))
+        extractor.close()
         out.seek(0)
         return extract_javascript(out, keywords, comment_tags, options)
 
 
+    def extract_html(fileobj, keywords, comment_tags, options):
+        """Extracts translatable texts from templates.
+
+        We simplify white-space found in translatable texts collected
+        via the ``gettext`` function (which is what the ``trans``
+        directives use), otherwise we would have near duplicates
+        (e.g. admin.html, prefs.html).
+
+        We assume the template function ``gettext`` will do the same
+        before trying to fetch the translation from the catalog.
+
+        """
+        if fileobj:
+            extractor = jinja2_extractor
+            fileobj.seek(0)
+            for m in extractor(fileobj, keywords, comment_tags, options):
+                # lineno, func, message, comments = m
+                if m[1] in ('gettext', None):
+                    # Jinja2 trans
+                    yield m[0], m[1], simplify_message(m[2]), m[3]
+                else:
+                    yield m
+
+
+    extract_text = extract_html
+
+
     class generate_messages_js(Command):
         """Generating message javascripts command for use ``setup.py`` scripts.
         """
@@ -292,7 +369,7 @@ try:
                 if self.output_file:
                     js_files.append(self.output_file)
                 else:
-                    js_files.append(js_path(self.output_dir, locale))
+                    js_files.append(js_path(self.output_dir, self.locale))
 
             if not mo_files:
                 raise DistutilsOptionError('no compiled catalogs found')
@@ -302,14 +379,14 @@ try:
 
             for idx, (locale, mo_file) in enumerate(mo_files):
                 js_file = js_files[idx]
-                log.info('generating messages javascript %r to %r',
-                         mo_file, js_file)
+                distlog.info('generating messages javascript %r to %r',
+                             mo_file, js_file)
 
                 with open(mo_file, 'rb') as infile:
                     t = Translations(infile, self.domain)
                     catalog = t._catalog
 
-                with open(js_file, 'w') as outfile:
+                with open(js_file, 'w', encoding='utf-8') as outfile:
                     write_js(outfile, catalog, self.domain, locale)
 
 
@@ -341,15 +418,13 @@ try:
 
         def run(self):
             for filename in self._get_po_files():
-                log.info('checking catalog %s', filename)
-                f = open(filename)
-                try:
+                distlog.info('checking catalog %s', filename)
+                with open(filename, 'rb') as f:
                     catalog = read_po(f, domain=self.domain)
-                finally:
-                    f.close()
                 for message in catalog:
                     for error in self._check_message(catalog, message):
-                        log.warn('%s:%d: %s', filename, message.lineno, error)
+                        distlog.warn('%s:%d: %s', filename, message.lineno,
+                                     error)
 
         def _get_po_files(self):
             if self.input_file:
@@ -368,58 +443,66 @@ try:
             return sorted(files)
 
         def _check_message(self, catalog, message):
-            errors = [e for e in message.check(catalog)]
-            try:
-                check_genshi_markup(catalog, message)
-            except TranslationError as e:
-                errors.append(e)
-            return errors
+            for e in message.check(catalog):
+                yield e
+            for e in check_markup(catalog, message):
+                yield e
 
+    def check_markup(catalog, message):
+        """Verify markups in the translation."""
+        def to_array(value):
+            if not isinstance(value, (list, tuple)):
+                value = (value,)
+            return value
+        msgids = to_array(message.id)
+        msgstrs = to_array(message.string)
+        for msgid_idx, msgid in enumerate(msgids):
+            msgid_name = 'msgid' if msgid_idx == 0 else 'msgid_plural'
+            for msgstr_idx, msgstr in enumerate(msgstrs):
+                if msgid and msgstr and msgid != msgstr:
+                    msgstr_name = 'msgstr' if len(msgids) == 1 else \
+                                  'msgstr[%d]' % msgstr_idx
+                    for e in _check_markup_0(msgid, msgid_name, msgstr,
+                                             msgstr_name):
+                        yield e
 
-    def check_genshi_markup(catalog, message):
-        """Verify the genshi markups in the translation."""
-        msgids = message.id
-        if not isinstance(msgids, (list, tuple)):
-            msgids = (msgids,)
-        msgstrs = message.string
-        if not isinstance(msgstrs, (list, tuple)):
-            msgstrs = (msgstrs,)
+    def _check_markup_0(msgid, msgid_name, msgstr, msgstr_name):
+        from xml.etree import ElementTree
 
-        # check using genshi-markup
-        if not _GENSHI_MARKUP_SEARCH(msgids[0]):
+        def count_tags(text):
+            text = '<html>\n%s\n</html>' % text.encode('utf-8')
+            counts = {}
+            for event in ElementTree.iterparse(io.BytesIO(text)):
+                tag = event[1].tag
+                counts.setdefault(tag, 0)
+                counts[tag] += 1
+            counts['html'] -= 1
+            return counts
+
+        try:
+            msgid_counts = count_tags(msgid)
+        except ElementTree.ParseError:
+            return
+        try:
+            msgstr_counts = count_tags(msgstr)
+        except ElementTree.ParseError as e:
+            yield TranslationError(e)
             return
 
-        for msgid, msgstr in izip(msgids, msgstrs):
-            if msgstr:
-                _validate_genshi_markup(msgid, msgstr)
-
-
-    def _validate_genshi_markup(markup, alternative):
-        indices_markup = _parse_genshi_markup(markup)
-        indices_alternative = _parse_genshi_markup(alternative)
-        indices = indices_markup - indices_alternative
-        if indices:
-            raise TranslationError(
-                'genshi markups are unbalanced %s' % \
-                ' '.join(['[%d:]' % idx for idx in indices]))
-
-
-    def _parse_genshi_markup(message):
-        from genshi.filters.i18n import parse_msg
-        try:
-            return set([idx for idx, text in parse_msg(message)
-                            if idx > 0])
-        except Exception as e:
-            raise TranslationError('cannot parse message (%s: %s)' % \
-                                   (e.__class__.__name__, unicode(e)))
-
+        for tag in (set(msgid_counts) | set(msgstr_counts)):
+            msgid_count = msgid_counts.get(tag, 0)
+            msgstr_count = msgstr_counts.get(tag, 0)
+            if msgid_count != msgstr_count:
+                yield TranslationError(
+                    "mismatched '%s' tag between %s and %s (%d != %d)" %
+                    (tag, msgid_name, msgstr_name, msgid_count, msgstr_count))
 
     def write_js(fileobj, catalog, domain, locale):
         from trac.util.presentation import to_json
         data = {'domain': domain, 'locale': locale}
 
         messages = {}
-        for msgid, msgstr in catalog.iteritems():
+        for msgid, msgstr in catalog.items():
             if isinstance(msgid, (list, tuple)):
                 messages.setdefault(msgid[0], {})
                 messages[msgid[0]][msgid[1]] = msgstr
@@ -438,11 +521,14 @@ try:
                         data['plural_expr'] = pluralexpr(val)
                         break
         data['messages'] = messages
+        data = to_json(data)
+        if isinstance(data, bytes):
+            data = str(data, 'utf-8')
 
         fileobj.write('// Generated messages javascript file '
                       'from compiled MO file\n')
         fileobj.write('babel.Translations.load(')
-        fileobj.write(to_json(data).encode('utf-8'))
+        fileobj.write(data)
         fileobj.write(').install();\n')
 
     def pluralexpr(forms):
@@ -532,7 +618,6 @@ try:
             'check_catalog_tracini': check_catalog,
         }
 
-
 except ImportError:
     def get_l10n_cmdclass():
         return
