@@ -10,10 +10,14 @@
     return label;
   }
 
+  function createTextHtml(name, size) {
+    return $.htmlFormat('<input type="text" name="$1" size="$2" />',
+                        name, size);
+  }
+
   // Create an <input type="text">
   function createText(name, size) {
-    return $($.htmlFormat('<input type="text" name="$1" size="$2">',
-                          name, size));
+    return $(createTextHtml(name, size));
   }
 
   // Create an <input type="checkbox">
@@ -128,22 +132,9 @@
       return false;
     });
 
-    // Enable only clicked checkbox and clear others
-    $filters.on("click", "td :checkbox, td :checkbox + label", function(event) {
-      if (!event.metaKey && !event.altKey)
-        return;
-      var clicked = this.tagName === "LABEL" ?
-                    document.getElementById(this.htmlFor) : this;
-      $('input[name="' + clicked.name + '"]', $filters).prop("checked", false);
-      $(clicked).prop("checked", true);
-      if (this.tagName === "LABEL") {
-        return false;
-      }
-    }).mousedown(function(event) {
-      if (event.metaKey || event.altKey) {
-        event.preventDefault(); // Prevent border on Firefox.
-      }
-    });
+    // Select only clicked checkbox and clear others
+    $filters.exclusiveOnClick("td :checkbox, td :checkbox + label");
+    $("fieldset#columns").exclusiveOnClick("label");
 
     // Make the submit buttons for removing filters client-side triggers
     $filters.find("input[type='submit'][name^='rm_filter_']").each(function() {
@@ -221,20 +212,18 @@
             .append(createRadio(propertyName, "0", propertyName + "_off"))
             .append(" ").append(createLabel(_("no"), propertyName + "_off"));
         } else if (property.type == "time") {
-          var endElement = createText(propertyName + "_end", 14);
-          focusElement = createText(propertyName, 14);
+          var startElement = createTextHtml(propertyName, 14);
+          var endElement = createTextHtml(propertyName + "_end", 14);
+          td.append($.parseHTML(
+            _("<label>between %(start)s</label> <label>and %(end)s</label>",
+              {start: startElement, end: endElement})));
+          focusElement = td.find('input[name="' + propertyName + '"]');
           if (property.format == "datetime") {
-            focusElement.datetimepicker();
-            endElement.datetimepicker();
+            td.find('input').datetimepicker();
           } else if (property.format == "date" ||
                      property.format == "relative") {
-            focusElement.datepicker();
-            endElement.datepicker();
+            td.find('input').datepicker();
           }
-          td.append(createLabel(_("between"))).append(" ")
-            .append(focusElement).append(" ")
-            .append(createLabel(_("and"))).append(" ")
-            .append(endElement);
         }
         tr.append(td);
       } else {
@@ -425,9 +414,10 @@
 
     // Add a new column with checkboxes for each ticket.
     // Selecting a ticket marks it for inclusion in the batch.
-    if ($("tr td.id", $table).length > 0) {
-      $("tr th.id", $table).before($('<th class="sel">'));
-      $("tr td.id", $table).each(function() {
+    var $results_cells = $(".trac-query-results tr td.id", $table);
+    if ($results_cells.length > 0) {
+      $(".trac-query-heading tr th.id", $table).before($('<th class="sel">'));
+      $results_cells.each(function() {
         var tId = $(this).text().substring(1);
         $(this).before(
           $('<td class="sel">').append(
@@ -438,6 +428,10 @@
           )
         );
       });
+      $(".trac-query-summary tr td.id", $table).before($('<td>'));
+      $("tbody tr .trac-colspan", $table).each(function () {
+        $(this).attr('colspan', $(this).attr('colspan') + 1);
+      });
     }
 
     // Add a Select All checkbox at the top of the column.
