@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2018 Edgewall Software
+# Copyright (C) 2004-2021 Edgewall Software
 # Copyright (C) 2004 Daniel Lundin <daniel@edgewall.com>
 # Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2006-2007 Christian Boos <cboos@edgewall.org>
@@ -8,11 +8,11 @@
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Daniel Lundin <daniel@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
@@ -39,8 +39,8 @@ In some cases, only the `url` pointing to the file's content is
 actually needed, that's why we avoid to read the file's content when
 it's not needed.
 
-The actual `content` to be converted might be a `unicode` object, but
-it can also be the raw byte string (`str`) object, or simply an object
+The actual `content` to be converted might be a `str` object, but
+it can also be the raw byte string (`bytes`) object, or simply an object
 that can be `read()`.
 
 .. note:: (for plugin developers)
@@ -59,25 +59,22 @@ that can be `read()`.
   corresponding ticket (#3332 as well).
 """
 
+import io
+import mimetypes
 import re
-from StringIO import StringIO
 from collections import namedtuple
 
-from genshi import Markup, Stream
-from genshi.core import TEXT, START, END, START_NS, END_NS
-from genshi.builder import Fragment, tag
-from genshi.input import HTMLParser
-
 from trac.config import IntOption, ListOption, Option
 from trac.core import Component, ExtensionPoint, Interface, TracError, \
                       implements
 from trac.resource import Resource
 from trac.util import Ranges, content_disposition
+from trac.util.html import Fragment, Markup, tag
 from trac.util.text import exception_to_unicode, to_unicode
 from trac.util.translation import _, tag_
 
 
-__all__ = ['Context', 'Mimeview', 'RenderingContext', 'get_mimetype',
+__all__ = ['Mimeview', 'RenderingContext', 'get_mimetype',
            'is_binary', 'detect_unicode', 'content_to_unicode', 'ct_mimetype']
 
 
@@ -133,15 +130,6 @@ class RenderingContext(object):
         self.perm = perm(resource) if perm and resource else perm
         self._hints = None
 
-    @staticmethod
-    def from_request(*args, **kwargs):
-        """
-        :deprecated: since 1.0, use `web_context` instead. Will be removed
-                     in release 1.3.1.
-        """
-        from trac.web.chrome import web_context
-        return web_context(*args, **kwargs)
-
     def __repr__(self):
         path = []
         context = self
@@ -284,14 +272,6 @@ class RenderingContext(object):
             p = p.parent
         return p and p._hints
 
-
-class Context(RenderingContext):
-    """
-    :deprecated: since 1.0, use `RenderingContext` instead. `Context` is
-                 kept for compatibility and will be removed release 1.3.1.
-    """
-
-
 # Some common MIME types and their associated keywords and/or file extensions
 
 KNOWN_MIME_TYPES = {
@@ -318,6 +298,7 @@ KNOWN_MIME_TYPES = {
 
     'text/css':                'css',
     'text/html':               'html htm',
+    'text/html+jinja':         'jinja',
     'text/plain':              'txt TXT text README INSTALL '
                                'AUTHORS COPYING ChangeLog RELEASE',
     'text/xml':                'xml',
@@ -352,7 +333,7 @@ KNOWN_MIME_TYPES = {
     'text/x-verilog':          'v',
     'text/x-vhdl':             'vhd',
 }
-for t in KNOWN_MIME_TYPES.keys():
+for t in KNOWN_MIME_TYPES:
     types = KNOWN_MIME_TYPES[t].split()
     if t.startswith('text/x-'):
         types.append(t[len('text/x-'):])
@@ -372,8 +353,7 @@ for x in TEXT_X_TYPES.split():
 MIME_MAP = {}
 for t, exts in KNOWN_MIME_TYPES.items():
     MIME_MAP[t] = t
-    for e in exts:
-        MIME_MAP[e] = t
+    MIME_MAP.update((e, t) for e in exts)
 
 # Simple builtin autodetection from the content using a regexp
 MODE_RE = re.compile(r"""
@@ -381,7 +361,7 @@ MODE_RE = re.compile(r"""
     | \#!(?:[/\w.-_]+/)?(\w+)               # 2. look for regular shebang
     | -\*-\s*(?:mode:\s*)?([\w+-]+)\s*-\*-  # 3. look for Emacs' -*- mode -*-
     | vim:.*?(?:syntax|filetype|ft)=(\w+)   # 4. look for VIM's syntax=<n>
-    """, re.VERBOSE)
+    """.encode('utf-8'), re.VERBOSE)
 
 
 def get_mimetype(filename, content=None, mime_map=MIME_MAP,
@@ -391,10 +371,10 @@ def get_mimetype(filename, content=None, mime_map=MIME_MAP,
     `filename` is either a filename (the lookup will then use the suffix)
     or some arbitrary keyword.
 
-    `content` is either a `str` or an `unicode` string.
+    `content` is either a `bytes` or a `str` string.
     """
     # 0) mimetype from filename pattern (most specific)
-    for mimetype, regexp in mime_map_patterns.iteritems():
+    for mimetype, regexp in mime_map_patterns.items():
         if regexp.match(filename):
             return mimetype
     suffix = filename.split('.')[-1]
@@ -404,16 +384,20 @@ def get_mimetype(filename, content=None, mime_map=MIME_MAP,
     else:
         mimetype = None
         try:
-            import mimetypes
             # 2) mimetype from the suffix, using the `mimetypes` module
             mimetype = mimetypes.guess_type(filename)[0]
         except Exception:
             pass
         if not mimetype and content:
-            match = re.search(MODE_RE, content[:1000] + content[-1000:])
+            chunk = (content[:1000], content[-1000:])
+            if isinstance(content, str):
+                chunk = map(lambda v: v.encode('utf-8'), chunk)
+            chunk = b'\n\n'.join(chunk)
+            match = MODE_RE.search(chunk)
             if match:
-                mode = match.group(1) or match.group(2) or match.group(4) or \
-                    match.group(3).lower()
+                mode = str(match.group(1) or match.group(2) or
+                           match.group(4) or match.group(3).lower(),
+                           'iso-8859-1')
                 if mode in mime_map:
                     # 3) mimetype from the content, using the `MODE_RE`
                     return mime_map[mode]
@@ -432,46 +416,51 @@ def ct_mimetype(content_type):
 def is_binary(data):
     """Detect binary content by checking the first thousand bytes for zeroes.
 
-    Operate on either `str` or `unicode` strings.
+    Operate on either `bytes` or `str` strings.
     """
-    if isinstance(data, str) and detect_unicode(data):
-        return False
-    return '\0' in data[:1000]
+    if isinstance(data, bytes):
+        if detect_unicode(data):
+            return False
+        return data.find(0, 0, 1000) != -1
+    if isinstance(data, str):
+        return data.find('\0', 0, 1000) != -1
+    return False
 
 
 def detect_unicode(data):
     """Detect different unicode charsets by looking for BOMs (Byte Order Mark).
 
-    Operate obviously only on `str` objects.
+    Operate obviously only on `bytes` objects.
     """
-    if data.startswith('\xff\xfe'):
+    if data.startswith(b'\xff\xfe'):
         return 'utf-16-le'
-    elif data.startswith('\xfe\xff'):
+    elif data.startswith(b'\xfe\xff'):
         return 'utf-16-be'
-    elif data.startswith('\xef\xbb\xbf'):
+    elif data.startswith(b'\xef\xbb\xbf'):
         return 'utf-8'
     else:
         return None
 
 
 def content_to_unicode(env, content, mimetype):
-    """Retrieve an `unicode` object from a `content` to be previewed.
+    """Retrieve an `str` object from a `content` to be previewed.
 
     In case the raw content had an unicode BOM, we remove it.
 
     >>> from trac.test import EnvironmentStub
     >>> env = EnvironmentStub()
-    >>> content_to_unicode(env, u"\ufeffNo BOM! h\u00e9 !", '')
-    u'No BOM! h\\xe9 !'
-    >>> content_to_unicode(env, "\xef\xbb\xbfNo BOM! h\xc3\xa9 !", '')
-    u'No BOM! h\\xe9 !'
+    >>> content_to_unicode(env, "\ufeffNo BOM! h\u00e9 !", '')
+    'No BOM! h\\xe9 !'
+    >>> content_to_unicode(env, bytes([0xEF, 0xBB, 0xBF]) + b'No BOM! h' +
+    ...                         bytes([0xC3, 0xA9]) + b' !', '')
+    'No BOM! h\\xe9 !'
 
     """
     mimeview = Mimeview(env)
     if hasattr(content, 'read'):
         content = content.read(mimeview.max_preview_size)
     u = mimeview.to_unicode(content, mimetype)
-    if u and u[0] == u'\ufeff':
+    if u and u[0] == '\ufeff':
         u = u[1:]
     return u
 
@@ -532,7 +521,7 @@ class IHTMLPreviewRenderer(Interface):
         `RenderingContext`.
 
         The `content` might be:
-         * a `str` object
+         * a `bytes` object
          * an `unicode` string
          * any object with a `read` method, returning one of the above
 
@@ -603,8 +592,8 @@ class IContentConverter(Interface):
         represented by key. Returns a tuple in the form (content,
         output_mime_type) or None if conversion is not possible.
 
-        content must be a `str` instance or an iterable instance which
-        iterates `str` instances."""
+        content must be a `bytes` instance or an iterable instance which
+        iterates `bytes` instances."""
 
 
 class Content(object):
@@ -618,7 +607,7 @@ class Content(object):
         if size == 0:
             return ''
         if self.content is None:
-            self.content = StringIO(self.input.read(self.max_size))
+            self.content = io.BytesIO(self.input.read(self.max_size))
         return self.content.read(size)
 
     def reset(self):
@@ -723,15 +712,27 @@ class Mimeview(Component):
             output = conversion.converter.convert_content(req, mimetype,
                                                           content,
                                                           conversion.key)
-            if output:
-                content, content_type = output
-                if iterable:
-                    if isinstance(content, basestring):
-                        content = (content,)
-                else:
-                    if not isinstance(content, basestring):
-                        content = ''.join(content)
-                return content, content_type, conversion.extension
+            if not output:
+                continue
+            content, content_type = output
+            is_iterable = not isinstance(content, (str, bytes))
+            if iterable:
+                if not is_iterable:
+                    content = (content,)
+            else:
+                if is_iterable:
+                    content = iter(content)
+                    try:
+                        first = next(content)
+                    except StopIteration:
+                        content = b''  # the iterable is empty
+                    else:
+                        join = ''.join if isinstance(first, str) else b''.join
+                        def g():
+                            yield first
+                            yield from content
+                        content = join(g())
+            return content, content_type, conversion.extension
         raise TracError(
             _("No available MIME conversions from %(old)s to %(new)s",
               old=mimetype, new=key))
@@ -761,8 +762,6 @@ class Mimeview(Component):
         """
         if not content:
             return ''
-        if not isinstance(context, RenderingContext):
-            raise TypeError("RenderingContext expected (since 0.11)")
 
         # Ensure we have a MIME type for this content
         full_mimetype = mimetype
@@ -781,7 +780,7 @@ class Mimeview(Component):
             qr = renderer.get_quality_ratio(mimetype)
             if qr > 0:
                 candidates.append((qr, renderer))
-        candidates.sort(lambda x, y: cmp(y[0], x[0]))
+        candidates.sort(key=lambda item: -item[0])
 
         # Wrap file-like object so that it can be read multiple times
         if hasattr(content, 'read'):
@@ -795,35 +794,43 @@ class Mimeview(Component):
                 continue  # skip non-source renderers in force_source mode
             if isinstance(content, Content):
                 content.reset()
+            ann_names = ', '.join(annotations) if annotations else \
+                       'no annotations'
+            self.log.debug('Trying to render HTML preview using %s [%s]',
+                           renderer.__class__.__name__, ann_names)
+
+            # check if we need to perform a tab expansion
+            rendered_content = content
+            if getattr(renderer, 'expand_tabs', False):
+                if expanded_content is None:
+                    content = content_to_unicode(self.env, content,
+                                                 full_mimetype)
+                    expanded_content = content.expandtabs(self.tab_width)
+                rendered_content = expanded_content
             try:
-                ann_names = ', '.join(annotations) if annotations else \
-                           'no annotations'
-                self.log.debug('Trying to render HTML preview using %s [%s]',
-                               renderer.__class__.__name__, ann_names)
-
-                # check if we need to perform a tab expansion
-                rendered_content = content
-                if getattr(renderer, 'expand_tabs', False):
-                    if expanded_content is None:
-                        content = content_to_unicode(self.env, content,
-                                                     full_mimetype)
-                        expanded_content = content.expandtabs(self.tab_width)
-                    rendered_content = expanded_content
-
                 result = renderer.render(context, full_mimetype,
                                          rendered_content, filename, url)
+            except Exception as e:
+                self.log.warning('HTML preview using %s with %r failed: %s',
+                                 renderer.__class__.__name__, context,
+                                 exception_to_unicode(e, traceback=True))
+                if context.req and not context.get_hint('disable_warnings'):
+                    from trac.web.chrome import add_warning
+                    add_warning(context.req,
+                        _("HTML preview using %(renderer)s failed (%(err)s)",
+                          renderer=renderer.__class__.__name__,
+                          err=exception_to_unicode(e)))
+            else:
                 if not result:
                     continue
 
                 if not (force_source or getattr(renderer, 'returns_source',
                                                 False)):
                     # Direct rendering of content
-                    if isinstance(result, basestring):
-                        if not isinstance(result, unicode):
-                            result = to_unicode(result)
-                        return Markup(to_unicode(result))
+                    if isinstance(result, str):
+                        return Markup(result)
                     elif isinstance(result, Fragment):
-                        return result.generate()
+                        return result.__html__()
                     else:
                         return result
 
@@ -837,20 +844,9 @@ class Mimeview(Component):
                 else:
                     if isinstance(result, list):
                         result = Markup('\n').join(result)
-                    return tag.div(class_='code')(tag.pre(result)).generate()
+                    return tag.div(class_='code')(tag.pre(result))
 
-            except Exception as e:
-                self.log.warning('HTML preview using %s with %r failed: %s',
-                                 renderer.__class__.__name__, context,
-                                 exception_to_unicode(e, traceback=True))
-                if context.req and not context.get_hint('disable_warnings'):
-                    from trac.web.chrome import add_warning
-                    add_warning(context.req,
-                        _("HTML preview using %(renderer)s failed (%(err)s)",
-                          renderer=renderer.__class__.__name__,
-                          err=exception_to_unicode(e)))
-
-    def _render_source(self, context, stream, annotations):
+    def _render_source(self, context, lines, annotations):
         from trac.web.chrome import add_warning
         annotators, labels, titles = {}, {}, {}
         for annotator in self.annotators:
@@ -861,14 +857,10 @@ class Mimeview(Component):
                 annotators[atype] = annotator
         annotations = [a for a in annotations if a in annotators]
 
-        if isinstance(stream, list):
-            stream = HTMLParser(StringIO(u'\n'.join(stream)))
-        elif isinstance(stream, unicode):
-            text = stream
-            def linesplitter():
-                for line in text.splitlines(True):
-                    yield TEXT, line, (None, -1, -1)
-            stream = linesplitter()
+        if isinstance(lines, str):
+            lines = lines.splitlines(True)
+        # elif isinstance(lines, list):
+        #    pass # assume these are lines already
 
         annotator_datas = []
         for a in annotations:
@@ -887,15 +879,16 @@ class Mimeview(Component):
             return tag.tr(
                 [tag.th(labels[a], class_=a, title=titles[a])
                  for a in annotations] +
-                [tag.th(u'\xa0', class_='content')]
+                [tag.th('\xa0', class_='content')]
             )
 
         def _body_rows():
-            for idx, line in enumerate(_group_lines(stream)):
+            for idx, line in enumerate(lines):
                 row = tag.tr()
                 for annotator, data in annotator_datas:
                     if annotator:
-                        annotator.annotate_row(context, row, idx+1, line, data)
+                        annotator.annotate_row(context, row, idx + 1,
+                                               line, data)
                     else:
                         row.append(tag.td())
                 row.append(tag.td(line))
@@ -909,7 +902,7 @@ class Mimeview(Component):
     def get_charset(self, content='', mimetype=None):
         """Infer the character encoding from the `content` or the `mimetype`.
 
-        `content` is either a `str` or an `unicode` object.
+        `content` is either a `bytes` or a `str` object.
 
         The charset will be determined using this order:
          * from the charset information present in the `mimetype` argument
@@ -920,7 +913,7 @@ class Mimeview(Component):
             ctpos = mimetype.find('charset=')
             if ctpos >= 0:
                 return mimetype[ctpos + 8:].strip()
-        if isinstance(content, str):
+        if isinstance(content, bytes):
             utf = detect_unicode(content)
             if utf is not None:
                 return utf
@@ -949,7 +942,7 @@ class Mimeview(Component):
     def get_mimetype(self, filename, content=None):
         """Infer the MIME type from the `filename` or the `content`.
 
-        `content` is either a `str` or an `unicode` object.
+        `content` is either a `bytes` or a `str` object.
 
         Return the detected MIME type, augmented by the
         charset information (i.e. "<mimetype>; charset=..."),
@@ -961,7 +954,7 @@ class Mimeview(Component):
         charset = None
         if mimetype:
             charset = self.get_charset(content, mimetype)
-        if mimetype and charset and not 'charset' in mimetype:
+        if mimetype and charset and 'charset' not in mimetype:
             mimetype += '; charset=' + charset
         return mimetype
 
@@ -994,7 +987,7 @@ class Mimeview(Component):
         return False
 
     def to_unicode(self, content, mimetype=None, charset=None):
-        """Convert `content` (an encoded `str` object) to an `unicode` object.
+        """Convert `content` (an encoded `bytes` object) to a `str` object.
 
         This calls `trac.util.to_unicode` with the `charset` provided,
         or the one obtained by `Mimeview.get_charset()`.
@@ -1049,13 +1042,13 @@ class Mimeview(Component):
         if iterable:
             def encoder(content):
                 for chunk in content:
-                    if isinstance(chunk, unicode):
+                    if isinstance(chunk, str):
                         chunk = chunk.encode('utf-8')
                     yield chunk
             content = encoder(content)
             length = None
         else:
-            if isinstance(content, unicode):
+            if isinstance(content, str):
                 content = content.encode('utf-8')
             length = len(content)
         req.send_response(200)
@@ -1071,82 +1064,6 @@ class Mimeview(Component):
         raise RequestDone
 
 
-def _group_lines(stream):
-    space_re = re.compile('(?P<spaces> (?: +))|^(?P<tag><\w+.*?>)?( )')
-
-    def pad_spaces(match):
-        m = match.group('spaces')
-        if m:
-            div, mod = divmod(len(m), 2)
-            return div * u'\xa0 ' + mod * u'\xa0'
-        return (match.group('tag') or '') + u'\xa0'
-
-    def _generate():
-        stack = []
-        def _reverse():
-            for event in reversed(stack):
-                if event[0] is START:
-                    yield END, event[1][0], event[2]
-                else:
-                    yield END_NS, event[1][0], event[2]
-
-        for kind, data, pos in stream:
-            if kind is TEXT:
-                lines = data.split('\n')
-                if lines:
-                    # First element
-                    for e in stack:
-                        yield e
-                    yield kind, lines.pop(0), pos
-                    for e in _reverse():
-                        yield e
-                    # Subsequent ones, prefix with \n
-                    for line in lines:
-                        yield TEXT, '\n', pos
-                        for e in stack:
-                            yield e
-                        yield kind, line, pos
-                        for e in _reverse():
-                            yield e
-            else:
-                if kind is START or kind is START_NS:
-                    stack.append((kind, data, pos))
-                elif kind is END or kind is END_NS:
-                    stack.pop()
-                else:
-                    yield kind, data, pos
-
-    buf = []
-
-    # Fix the \n at EOF.
-    if not isinstance(stream, list):
-        stream = list(stream)
-    found_text = False
-
-    for i in range(len(stream)-1, -1, -1):
-        if stream[i][0] is TEXT:
-            e = stream[i]
-            # One chance to strip a \n
-            if not found_text and e[1].endswith('\n'):
-                stream[i] = (e[0], e[1][:-1], e[2])
-            if len(e[1]):
-                found_text = True
-                break
-    if not found_text:
-        raise StopIteration
-
-    for kind, data, pos in _generate():
-        if kind is TEXT and data == '\n':
-            yield Stream(buf[:])
-            del buf[:]
-        else:
-            if kind is TEXT:
-                data = space_re.sub(pad_spaces, data)
-            buf.append((kind, data, pos))
-    if buf:
-        yield Stream(buf[:])
-
-
 # -- Default annotators
 
 class LineNumberAnnotator(Component):
