@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2018 Edgewall Software
+# Copyright (C) 2003-2021 Edgewall Software
 # Copyright (C) 2003-2004 Jonas Borgström <jonas@edgewall.com>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 
@@ -28,7 +28,7 @@ from trac.util.text import quote_query_string
 from trac.util.translation import _
 from trac.web.api import IRequestHandler
 from trac.web.chrome import (INavigationContributor, ITemplateProvider,
-                             add_link, add_notice, add_stylesheet,
+                             accesskey, add_link, add_notice, add_stylesheet,
                              add_warning, web_context)
 from trac.wiki.api import IWikiSyntaxProvider
 from trac.wiki.formatter import extract_link
@@ -59,7 +59,7 @@ class SearchModule(Component):
                the `get_search_filters()` method, the first member of
                returned tuple. Once disabled, search filters can still
                be manually enabled by the user on the search page.
-               (''since 0.12'')""")
+               """)
 
     # INavigationContributor methods
 
@@ -69,7 +69,8 @@ class SearchModule(Component):
     def get_navigation_items(self, req):
         if 'SEARCH_VIEW' in req.perm:
             yield ('mainnav', 'search',
-                   tag.a(_('Search'), href=req.href.search(), accesskey=4))
+                   tag.a(_('Search'), href=req.href.search(),
+                         accesskey=accesskey(req, 4)))
 
     # IPermissionRequestor methods
 
@@ -83,13 +84,13 @@ class SearchModule(Component):
                is not None
 
     def process_request(self, req):
-        req.perm.assert_permission('SEARCH_VIEW')
+        req.perm.require('SEARCH_VIEW')
 
         if req.path_info == '/search/opensearch':
             return ('opensearch.xml', {},
-                    'application/opensearchdescription+xml')
+                    {'content_type': 'application/opensearchdescription+xml'})
 
-        query = req.args.getfirst('q')
+        query = req.args.get('q')
         available_filters = []
         for source in self.search_sources:
             available_filters.extend(source.get_search_filters(req) or [])
@@ -98,7 +99,8 @@ class SearchModule(Component):
         filters = self._get_selected_filters(req, available_filters)
         data = self._prepare_data(req, query, available_filters, filters)
         if query:
-            data['quickjump'] = self._check_quickjump(req, query)
+            noquickjump = req.args.getbool('noquickjump', False)
+            data['quickjump'] = self._check_quickjump(req, noquickjump, query)
             if query.startswith('!'):
                 query = query[1:]
 
@@ -107,9 +109,11 @@ class SearchModule(Component):
                 results = self._do_search(req, terms, filters)
                 if results:
                     data.update(self._prepare_results(req, filters, results))
+            if noquickjump and filters:
+                req.session['search.filters'] = ','.join(filters)
 
         add_stylesheet(req, 'common/css/search.css')
-        return 'search.html', data, None
+        return 'search.html', data
 
     # ITemplateProvider methods
 
@@ -146,9 +150,16 @@ class SearchModule(Component):
         """
         filters = [f[0] for f in available_filters if f[0] in req.args]
         if not filters:
-            filters = [f[0] for f in available_filters
-                       if f[0] not in self.default_disabled_filters and
-                       (len(f) < 3 or len(f) > 2 and f[2])]
+            session_filters = req.session.get('search.filters')
+            if session_filters:
+                available_filters_dict = dict([f[0:2]
+                                               for f in available_filters])
+                filters = [f for f in session_filters.split(',')
+                           if f in available_filters_dict]
+            else:
+                filters = [f[0] for f in available_filters
+                           if f[0] not in self.default_disabled_filters and
+                           (len(f) < 3 or len(f) > 2 and f[2])]
         return filters
 
     def _prepare_data(self, req, query, available_filters, filters):
@@ -157,9 +168,8 @@ class SearchModule(Component):
                             for f in available_filters],
                 'query': query, 'quickjump': None, 'results': []}
 
-    def _check_quickjump(self, req, kwd):
+    def _check_quickjump(self, req, noquickjump, kwd):
         """Look for search shortcuts"""
-        noquickjump = req.args.getbool('noquickjump', False)
         # Source quickjump  FIXME: delegate to ISearchSource.search_quickjump
         quickjump_href = None
         if kwd[0] == '/':
@@ -182,7 +192,8 @@ class SearchModule(Component):
                         'description': description}
             else:
                 help_url = req.href.wiki('TracSearch') + '#Quicksearches'
-                search_url = req.href.search(q=kwd, noquickjump=1)
+                search_url = req.href.search([('q', kwd), ('noquickjump', 1)])
+                # FIXME: use tag_
                 add_notice(req, Markup(_(
                     'You arrived here through the <a href="%(help_url)s">'
                     'quick-jump</a> search feature. To instead search for the '
