@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2020 Edgewall Software
+# Copyright (C) 2006-2021 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -842,7 +842,7 @@ class NavContribReportModuleEnabledTestCase(NavigationContributorTestCase):
 
     def setUp(self):
         self.env = EnvironmentStub()
-        super(NavContribReportModuleEnabledTestCase, self).setUp()
+        super().setUp()
 
     def test_user_has_no_perms(self):
         """No navigation item when user has neither REPORT_VIEW or
@@ -898,7 +898,7 @@ class NavContribReportModuleDisabledTestCase(NavigationContributorTestCase):
 
     def setUp(self):
         self.env = EnvironmentStub(disable=['trac.ticket.report.*'])
-        super(NavContribReportModuleDisabledTestCase, self).setUp()
+        super().setUp()
 
     def test_user_has_ticket_view(self):
         """Navigation item directs to QueryModule when ReportModule is
