@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2020 Edgewall Software
+# Copyright (C) 2006-2021 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -265,7 +265,7 @@ consequat. Duis a ..."""
 class MilestoneModulePermissionsTestCase(RequestHandlerPermissionsTestCaseBase):
 
     def setUp(self):
-        super(MilestoneModulePermissionsTestCase, self).setUp(MilestoneModule)
+        super().setUp(MilestoneModule)
 
     def test_milestone_notfound_with_milestone_create(self):
         self.grant_perm('anonymous', 'MILESTONE_VIEW')
