@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2020 Edgewall Software
+# Copyright (C) 2007-2021 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -78,6 +78,14 @@ class FragmentTestCase(unittest.TestCase):
     def test_unicode(self):
         self.assertEqual('<b>M</b>essäge', str(tag(tag.b('M'), 'essäge')))
 
+    def test_call(self):
+        t = tag()
+        self.assertEqual('<b>M</b>',
+                         str(t(tag.b('M'))))
+        t = tag()
+        self.assertEqual('<b>M</b>essäge',
+                         str(t(tag.b('M'), 'essäge')))
+
 
 class XMLElementTestCase(unittest.TestCase):
 
