@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2015 Edgewall Software
+# Copyright (C) 2015-2021 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import unittest
 
@@ -21,12 +21,32 @@ from trac.versioncontrol.admin import RepositoryAdminPanel
 
 class RepositoryAdminPanelTestCase(unittest.TestCase):
 
+    RepositoryConnector = None
+
+    @classmethod
+    def setUpClass(cls):
+        class RepositoryConnector(Component):
+            implements(IRepositoryConnector)
+
+            def get_supported_types(self):
+                yield 'RepositoryConnector', 1
+
+            def get_repository(self, repos_type, repos_dir, params):
+                pass
+
+        cls.RepositoryConnector = RepositoryConnector
+
     def setUp(self):
         self.env = EnvironmentStub(enable=('trac.versioncontrol.admin.*',))
 
     def tearDown(self):
         self.env.reset_db()
 
+    @classmethod
+    def tearDownClass(cls):
+        from trac.core import ComponentMeta
+        ComponentMeta.deregister(cls.RepositoryConnector)
+
     def test_panel_not_exists_when_no_repository_connectors(self):
         """Repositories admin panel is not present when there are
         no repository connectors enabled.
@@ -41,16 +61,7 @@ class RepositoryAdminPanelTestCase(unittest.TestCase):
         """Repositories admin panel is present when there are
         repository connectors enabled.
         """
-        class RepositoryConnector(Component):
-            implements(IRepositoryConnector)
-
-            def get_supported_types(self):
-                yield 'RepositoryConnector', 1
-
-            def get_repository(self, repos_type, repos_dir, params):
-                pass
-
-        self.env.enable_component(RepositoryConnector)
+        self.env.enable_component(self.RepositoryConnector)
         req = MockRequest(self.env)
         rap = RepositoryAdminPanel(self.env)
         panels = [panel for panel in rap.get_admin_panels(req)]
