@@ -1,31 +1,29 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2018 Edgewall Software
+# Copyright (C) 2003-2021 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2005-2006 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Christian Boos <cboos@edgewall.org>
 
 import re
 
-from genshi.builder import tag
-from genshi.core import Markup
-
 from trac.config import IntOption, ListOption
 from trac.core import *
 from trac.perm import IPermissionRequestor
 from trac.resource import ResourceNotFound
 from trac.util import Ranges
+from trac.util.html import Markup, tag
 from trac.util.text import to_unicode, wrap
 from trac.util.translation import _
 from trac.versioncontrol.api import (Changeset, NoSuchChangeset,
@@ -81,7 +79,7 @@ class LogModule(Component):
 
         mode = req.args.get('mode', 'stop_on_copy')
         path = req.args.get('path', '/')
-        rev = req.args.getfirst('rev')
+        rev = req.args.get('rev')
         stop_rev = req.args.get('stop_rev')
         revs = req.args.get('revs')
         format = req.args.get('format')
@@ -218,7 +216,6 @@ class LogModule(Component):
             graph.update(threads=threads, vertices=vertices, columns=columns,
                          colors=self.graph_colors,
                          line_width=0.04, dot_radius=0.1)
-            add_script(req, 'common/js/excanvas.js', ie_if='IE')
             add_script(req, 'common/js/log_graph.js')
             add_script_data(req, graph=graph)
 
@@ -246,7 +243,7 @@ class LogModule(Component):
                                if info[-1]['change'] else None
                 next_revranges = revranges.truncate(curr_revrange,
                                                     new_revrange)
-                next_revranges = unicode(next_revranges) or None
+                next_revranges = str(next_revranges) or None
             if next_revranges or not revranges:
                 older_revisions_href = make_log_href(
                     next_path, rev=next_rev, revs=next_revranges)
@@ -290,12 +287,13 @@ class LogModule(Component):
         }
 
         if format == 'changelog':
-            return 'revisionlog.txt', data, 'text/plain'
+            return 'revisionlog.txt', data, {'content_type': 'text/plain'}
         elif format == 'rss':
             data['context'] = web_context(req, 'source',
                                           path, parent=repos.resource,
                                           absurls=True)
-            return 'revisionlog.rss', data, 'application/rss+xml'
+            return ('revisionlog.rss', data,
+                    {'content_type': 'application/rss+xml'})
 
         item_ranges = []
         range = []
@@ -337,7 +335,7 @@ class LogModule(Component):
                                             href=next['href']),
                                       Markup(' &rarr;')))
 
-        return 'revisionlog.html', data, None
+        return 'revisionlog.html', data
 
     # IWikiSyntaxProvider methods
 
@@ -398,13 +396,14 @@ class LogModule(Component):
                     repos = rm.get_repository(reponame)
 
             if repos:
-                if 'LOG_VIEW' in formatter.perm(repos.resource):
+                path = path or '/'
+                if 'LOG_VIEW' in formatter.perm(repos.resource
+                                                .child('source', path)):
                     reponame = repos.reponame or None
-                    path = path or '/'
                     revranges = RevRanges(repos, revs)
                     if revranges.has_ranges():
                         href = formatter.href.log(reponame, path,
-                                                  revs=unicode(revranges))
+                                                  revs=str(revranges))
                     else:
                         # try to resolve if single rev
                         repos.normalize_rev(revs)
@@ -482,7 +481,7 @@ class RevRanges(object):
             return
 
         pairs = []
-        for rev in re.split(u',\u200b?', revs):
+        for rev in re.split(',\u200b?', revs):
             a = b = None
             if self._cset_range_re.match(rev):
                 for sep in ':-':
@@ -501,11 +500,10 @@ class RevRanges(object):
         self._reduce()
 
     def _reduce(self):
-        if all(isinstance(pair[0], (int, long)) and
-               isinstance(pair[1], (int, long))
+        if all(isinstance(pair[0], int) and isinstance(pair[1], int)
                for pair in self.pairs):
             try:
-                ranges = Ranges(unicode(self), reorder=True)
+                ranges = Ranges(str(self), reorder=True)
             except:
                 pass
             else:
@@ -529,8 +527,8 @@ class RevRanges(object):
     def __len__(self):
         return len(self.pairs)
 
-    def __unicode__(self):
+    def __str__(self):
         sep = '-' if self.repos.has_linear_changesets else ':'
-        return ','.join(sep.join(map(unicode, pair)) if pair[0] != pair[1]
-                                                     else unicode(pair[0])
+        return ','.join(sep.join(map(str, pair)) if pair[0] != pair[1]
+                                                 else str(pair[0])
                         for pair in self.pairs)
