@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2014 Edgewall Software
+# Copyright (C) 2014-2021 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/.
+# history and logs, available at https://trac.edgewall.org/.
 
 import unittest
 
@@ -25,7 +25,7 @@ class ChangesetModuleTestCase(unittest.TestCase):
         self.cm = ChangesetModule(self.env)
 
     def test_default_repository_not_configured(self):
-        """Test for regression of http://trac.edgewall.org/ticket/11599."""
+        """Test for regression of https://trac.edgewall.org/ticket/11599."""
         req = MockRequest(self.env, args={'new_path': '/'})
         self.assertRaises(TracError, self.cm.process_request, req)
 
