@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2014-2020 Edgewall Software
+# Copyright (C) 2014-2021 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -130,7 +130,7 @@ class MockChangeset(Changeset):
 class MockNode(Node):
 
     def __init__(self, repos, path, rev, kind):
-        super(MockNode, self).__init__(repos, path, rev, kind)
+        super().__init__(repos, path, rev, kind)
         self.created_path = path
         self.created_rev = rev
 
@@ -171,7 +171,7 @@ class MockNode(Node):
 class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
 
     def setUp(self):
-        self._super = super(LogModuleTestCase, self)
+        self._super = super()
         self._super.setUp(LogModule)
         provider = DbRepositoryProvider(self.env)
         provider.add_repository('mock', '/', mock_repotype)
