@@ -5,11 +5,11 @@
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Allan Saddi <allan@saddi.com>
 
@@ -39,11 +39,11 @@ variable FCGI_FORCE_CGI to "Y" or "y".
 __author__ = 'Allan Saddi <allan@saddi.com>'
 __version__ = '$Revision: 2025 $'
 
+import io
 import sys
 import os
 import signal
 import struct
-import cStringIO as StringIO
 import select
 import socket
 import errno
@@ -121,9 +121,8 @@ if __debug__:
             return
 
         try:
-            f = open(DEBUGLOG, 'a')
-            f.write('%sfcgi: %s\n' % (time.ctime()[4:-4], msg))
-            f.close()
+            with open(DEBUGLOG, 'a', encoding='utf-8') as f:
+                f.write('%sfcgi: %s\n' % (time.ctime()[4:-4], msg))
         except:
             pass
 
@@ -230,12 +229,14 @@ class InputStream(object):
     def __iter__(self):
         return self
 
-    def next(self):
+    def __next__(self):
         r = self.readline()
         if not r:
             raise StopIteration
         return r
 
+    next = __next__
+
     def add_data(self, data):
         if not data:
             self._eof = True
@@ -250,7 +251,7 @@ class MultiplexedInputStream(InputStream):
     (the consumer) are running in different threads.
     """
     def __init__(self, conn):
-        super(MultiplexedInputStream, self).__init__(conn)
+        super().__init__(conn)
 
         # Arbitrates access to this InputStream (it's used simultaneously
         # by a Request and its owning Connection object).
@@ -266,21 +267,21 @@ class MultiplexedInputStream(InputStream):
     def read(self, n=-1):
         self._lock.acquire()
         try:
-            return super(MultiplexedInputStream, self).read(n)
+            return super().read(n)
         finally:
             self._lock.release()
 
     def readline(self, length=None):
         self._lock.acquire()
         try:
-            return super(MultiplexedInputStream, self).readline(length)
+            return super().readline(length)
         finally:
             self._lock.release()
 
     def add_data(self, data):
         self._lock.acquire()
         try:
-            super(MultiplexedInputStream, self).add_data(data)
+            super().add_data(data)
             self._lock.notify()
         finally:
             self._lock.release()
@@ -414,7 +415,7 @@ def decode_pair(s, pos=0):
     value = s[pos:pos+valueLength]
     pos += valueLength
 
-    return (pos, (name, value))
+    return pos, (name, value)
 
 def encode_pair(name, value):
     """
@@ -426,13 +427,13 @@ def encode_pair(name, value):
     if nameLength < 128:
         s = chr(nameLength)
     else:
-        s = struct.pack('!L', nameLength | 0x80000000L)
+        s = struct.pack('!L', nameLength | 0x80000000)
 
     valueLength = len(value)
     if valueLength < 128:
         s += chr(valueLength)
     else:
-        s += struct.pack('!L', valueLength | 0x80000000L)
+        s += struct.pack('!L', valueLength | 0x80000000)
 
     return s + name + value
 
@@ -461,7 +462,7 @@ class Record(object):
         while length:
             try:
                 data = sock.recv(length)
-            except socket.error, e:
+            except socket.error as e:
                 if e[0] == errno.EAGAIN:
                     select.select([sock], [], [])
                     continue
@@ -518,7 +519,7 @@ class Record(object):
         while length:
             try:
                 sent = sock.send(data)
-            except socket.error, e:
+            except socket.error as e:
                 if e[0] == errno.EAGAIN:
                     select.select([], [sock], [])
                     continue
@@ -582,11 +583,11 @@ class Request(object):
         try:
             self._flush()
             self._end(appStatus, protocolStatus)
-        except socket.error, e:
+        except socket.error as e:
             if e[0] != errno.EPIPE:
                 raise
 
-    def _end(self, appStatus=0L, protocolStatus=FCGI_REQUEST_COMPLETE):
+    def _end(self, appStatus=0, protocolStatus=FCGI_REQUEST_COMPLETE):
         self._conn.end_request(self, appStatus, protocolStatus)
 
     def _flush(self):
@@ -607,9 +608,9 @@ class CGIRequest(Request):
         self.stdin = sys.stdin
         self.stdout = StdoutWrapper(sys.stdout) # Oh, the humanity!
         self.stderr = sys.stderr
-        self.data = StringIO.StringIO()
+        self.data = io.BytesIO()
 
-    def _end(self, appStatus=0L, protocolStatus=FCGI_REQUEST_COMPLETE):
+    def _end(self, appStatus=0, protocolStatus=FCGI_REQUEST_COMPLETE):
         sys.exit(appStatus)
 
     def _flush(self):
@@ -658,7 +659,7 @@ class Connection(object):
                 self.process_input()
             except EOFError:
                 break
-            except (select.error, socket.error), e:
+            except (select.error, socket.error) as e:
                 if e[0] == errno.EBADF: # Socket was closed by Request.
                     break
                 raise
@@ -708,7 +709,7 @@ class Connection(object):
         """
         rec.write(self._sock)
 
-    def end_request(self, req, appStatus=0L,
+    def end_request(self, req, appStatus=0,
                     protocolStatus=FCGI_REQUEST_COMPLETE, remove=True):
         """
         End a Request.
@@ -757,7 +758,7 @@ class Connection(object):
 
         if not self._multiplexed and self._requests:
             # Can't multiplex requests.
-            self.end_request(req, 0L, FCGI_CANT_MPX_CONN, remove=False)
+            self.end_request(req, 0, FCGI_CANT_MPX_CONN, remove=False)
         else:
             self._requests[inrec.requestId] = req
 
@@ -820,7 +821,7 @@ class MultiplexedConnection(Connection):
     _inputStreamClass = MultiplexedInputStream
 
     def __init__(self, sock, addr, server):
-        super(MultiplexedConnection, self).__init__(sock, addr, server)
+        super().__init__(sock, addr, server)
 
         # Used to arbitrate access to self._requests.
         lock = threading.RLock()
@@ -836,7 +837,7 @@ class MultiplexedConnection(Connection):
             self._lock.wait()
         self._lock.release()
 
-        super(MultiplexedConnection, self)._cleanupSocket()
+        super()._cleanupSocket()
 
     def writeRecord(self, rec):
         # Must use locking to prevent intermingling of Records from different
@@ -848,11 +849,11 @@ class MultiplexedConnection(Connection):
         finally:
             self._lock.release()
 
-    def end_request(self, req, appStatus=0L,
+    def end_request(self, req, appStatus=0,
                     protocolStatus=FCGI_REQUEST_COMPLETE, remove=True):
         self._lock.acquire()
         try:
-            super(MultiplexedConnection, self).end_request(req, appStatus,
+            super().end_request(req, appStatus,
                                                            protocolStatus,
                                                            remove)
             self._lock.notify()
@@ -862,14 +863,14 @@ class MultiplexedConnection(Connection):
     def _do_begin_request(self, inrec):
         self._lock.acquire()
         try:
-            super(MultiplexedConnection, self)._do_begin_request(inrec)
+            super()._do_begin_request(inrec)
         finally:
             self._lock.release()
 
     def _do_abort_request(self, inrec):
         self._lock.acquire()
         try:
-            super(MultiplexedConnection, self)._do_abort_request(inrec)
+            super()._do_abort_request(inrec)
         finally:
             self._lock.release()
 
@@ -879,21 +880,21 @@ class MultiplexedConnection(Connection):
     def _do_params(self, inrec):
         self._lock.acquire()
         try:
-            super(MultiplexedConnection, self)._do_params(inrec)
+            super()._do_params(inrec)
         finally:
             self._lock.release()
 
     def _do_stdin(self, inrec):
         self._lock.acquire()
         try:
-            super(MultiplexedConnection, self)._do_stdin(inrec)
+            super()._do_stdin(inrec)
         finally:
             self._lock.release()
 
     def _do_data(self, inrec):
         self._lock.acquire()
         try:
-            super(MultiplexedConnection, self)._do_data(inrec)
+            super()._do_data(inrec)
         finally:
             self._lock.release()
 
@@ -984,7 +985,7 @@ class Server(object):
                                  socket.SOCK_STREAM)
             try:
                 sock.getpeername()
-            except socket.error, e:
+            except socket.error as e:
                 if e[0] == errno.ENOTSOCK:
                     # Not a socket, assume CGI context.
                     isFCGI = False
@@ -1057,8 +1058,8 @@ class Server(object):
         """
         web_server_addrs = os.environ.get('FCGI_WEB_SERVER_ADDRS')
         if web_server_addrs is not None:
-            web_server_addrs = map(lambda x: x.strip(),
-                                   web_server_addrs.split(','))
+            web_server_addrs = list(map(lambda x: x.strip(),
+                                        web_server_addrs.split(',')))
 
         sock = self._setupSocket()
 
@@ -1071,7 +1072,7 @@ class Server(object):
         while self._keepGoing:
             try:
                 r, w, e = select.select([sock], [], [], timeout)
-            except select.error, e:
+            except select.error as e:
                 if e[0] == errno.EINTR:
                     continue
                 raise
@@ -1079,7 +1080,7 @@ class Server(object):
             if r:
                 try:
                     clientSock, addr = sock.accept()
-                except socket.error, e:
+                except socket.error as e:
                     if e[0] in (errno.EINTR, errno.EAGAIN):
                         continue
                     raise
@@ -1125,7 +1126,7 @@ class Server(object):
         is passed at initialization time, this must be implemented by
         a subclass.
         """
-        raise NotImplementedError, self.__class__.__name__ + '.handler'
+        raise NotImplementedError(self.__class__.__name__ + '.handler')
 
     def error(self, req):
         """
@@ -1150,9 +1151,9 @@ class WSGIServer(Server):
 
         Set multithreaded to False if your application is not MT-safe.
         """
-        if kw.has_key('handler'):
+        if 'handler' in kw:
             del kw['handler'] # Doesn't make sense to let this through
-        super(WSGIServer, self).__init__(**kw)
+        super().__init__(**kw)
 
         if environ is None:
             environ = {}
@@ -1191,7 +1192,9 @@ class WSGIServer(Server):
         environ['wsgi.multiprocess'] = True
         environ['wsgi.run_once'] = isinstance(req, CGIRequest)
 
-        if environ.get('HTTPS', 'off') in ('on', '1'):
+        if environ.get('HTTPS', '').lower() in ('yes', 'on', '1'):
+            environ['wsgi.url_scheme'] = 'https'
+        elif environ.get('HTTP_X_FORWARDED_PROTO', '').lower() == 'https':
             environ['wsgi.url_scheme'] = 'https'
         else:
             environ['wsgi.url_scheme'] = 'http'
@@ -1233,8 +1236,7 @@ class WSGIServer(Server):
             if exc_info:
                 try:
                     if headers_sent:
-                        # Re-raise if too late
-                        raise exc_info[0], exc_info[1], exc_info[2]
+                        raise exc_info[1]  # Re-raise if too late
                 finally:
                     exc_info = None # avoid dangling circular ref
             else:
@@ -1267,7 +1269,7 @@ class WSGIServer(Server):
                 finally:
                     if hasattr(result, 'close'):
                         result.close()
-            except socket.error, e:
+            except socket.error as e:
                 if e[0] != errno.EPIPE:
                     raise # Don't let EPIPE propagate beyond server
         finally:
@@ -1278,9 +1280,9 @@ class WSGIServer(Server):
 
     def _sanitizeEnv(self, environ):
         """Ensure certain values are present, if required by WSGI."""
-        if not environ.has_key('SCRIPT_NAME'):
+        if 'SCRIPT_NAME' not in environ:
             environ['SCRIPT_NAME'] = ''
-        if not environ.has_key('PATH_INFO'):
+        if 'PATH_INFO' not in environ:
             environ['PATH_INFO'] = ''
 
         # If any of these are missing, it probably signifies a broken
@@ -1289,7 +1291,7 @@ class WSGIServer(Server):
                              ('SERVER_NAME', 'localhost'),
                              ('SERVER_PORT', '80'),
                              ('SERVER_PROTOCOL', 'HTTP/1.0')]:
-            if not environ.has_key(name):
+            if name not in environ:
                 environ['wsgi.errors'].write('%s: missing FastCGI param %s '
                                              'required by WSGI!\n' %
                                              (self.__class__.__name__, name))
@@ -1300,26 +1302,24 @@ if __name__ == '__main__':
         """Probably not the most efficient example."""
         import cgi
         start_response('200 OK', [('Content-Type', 'text/html')])
-        yield '<html><head><title>Hello World!</title></head>\n' \
-              '<body>\n' \
-              '<p>Hello World!</p>\n' \
-              '<table border="1">'
-        names = environ.keys()
-        names.sort()
-        for name in names:
-            yield '<tr><td>%s</td><td>%s</td></tr>\n' % (
-                name, cgi.escape(`environ[name]`))
+        yield b'<html><head><title>Hello World!</title></head>\n' \
+              b'<body>\n' \
+              b'<p>Hello World!</p>\n' \
+              b'<table border="1">'
+        for name in sorted(environ):
+            yield b'<tr><td>%s</td><td>%s</td></tr>\n' % \
+                  (name, cgi.escape(environ[name]))
 
         form = cgi.FieldStorage(fp=environ['wsgi.input'], environ=environ,
                                 keep_blank_values=1)
         if form.list:
-            yield '<tr><th colspan="2">Form data</th></tr>'
+            yield b'<tr><th colspan="2">Form data</th></tr>'
 
         for field in form.list:
-            yield '<tr><td>%s</td><td>%s</td></tr>\n' % (
-                field.name, field.value)
+            yield b'<tr><td>%s</td><td>%s</td></tr>\n' % \
+                  (field.name, field.value)
 
-        yield '</table>\n' \
-              '</body></html>\n'
+        yield b'</table>\n' \
+              b'</body></html>\n'
 
     WSGIServer(test_app).run()
