@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2018 Edgewall Software
+# Copyright (C) 2005-2021 Edgewall Software
 # Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
@@ -18,12 +18,9 @@ from fnmatch import fnmatchcase
 from itertools import groupby
 import fnmatch
 import inspect
+import io
 import os
 import re
-from StringIO import StringIO
-
-from genshi.builder import tag
-from genshi.core import Markup
 
 from trac.core import *
 from trac.resource import (
@@ -32,7 +29,7 @@ from trac.resource import (
 )
 from trac.util import as_int
 from trac.util.datefmt import format_date, from_utimestamp, user_time
-from trac.util.html import escape, find_element
+from trac.util.html import Markup, escape, find_element, tag
 from trac.util.presentation import separated
 from trac.util.text import unicode_quote, to_unicode, stripws
 from trac.util.translation import _, dgettext, cleandoc_, tag_
@@ -52,6 +49,13 @@ class WikiMacroBase(Component):
     On usage error, the `MacroError` or `ProcessorError` exception should be
     raised, to ensure proper display of the error message in the rendered
     wiki content.
+
+    The `_description` attribute contains the macro help.
+
+    Set the `hide_from_macro_index` attribute to `True` to prevent
+    displaying help in the macro index (`[[MacroList]]`). If the
+    default value of `False` and the `_description` is empty,
+    "No documentation found" will be displayed.
     """
 
     implements(IWikiMacroProvider)
@@ -63,6 +67,9 @@ class WikiMacroBase(Component):
     #: A macro description
     _description = None
 
+    #: Hide from macro index
+    hide_from_macro_index = False
+
     def get_macros(self):
         """Yield the name of the macro based on the class name."""
         name = self.__class__.__name__
@@ -72,12 +79,15 @@ class WikiMacroBase(Component):
 
     def get_macro_description(self, name):
         """Return the subclass's gettext domain and macro description"""
+        if self.hide_from_macro_index:
+            return None
         domain, description = self._domain, self._description
         if description:
             return (domain, description) if domain else description
         # For pre-0.12 compatibility
-        doc = inspect.getdoc(self.__class__)
-        return to_unicode(doc) if doc else ''
+        # don't use inspect.getdoc() to avoid retreiving from the inheritance
+        # hierarchy
+        return inspect.cleandoc(self.__doc__) if self.__doc__ else ''
 
     def parse_macro(self, parser, name, content):
         raise NotImplementedError
@@ -205,10 +215,12 @@ class TitleIndexMacro(WikiMacroBase):
                   grouped (path element, page_name) entries
                 - `nodes` is a list of nodes or leaves
             """
+            def keyfn(args):
+                elts, name = args
+                return elts[0] if elts else ''
             groups = []
 
-            for key, grouper in groupby(entries, lambda (elts, name):
-                                                    elts[0] if elts else ''):
+            for key, grouper in groupby(entries, keyfn):
                 # remove key from path_elements in grouped entries for further
                 # grouping
                 grouped_entries = [(path_elements[1:], page_name)
@@ -249,10 +261,12 @@ class TitleIndexMacro(WikiMacroBase):
                 - `page` is a page name (if one exists for that node)
                 - `nodes` is a list of nodes or leaves
             """
+            def keyfn(args):
+                elts, name = args
+                return elts[0] if elts else ''
             groups = []
 
-            for key, grouper in groupby(entries, lambda (elts, name):
-                                                    elts[0] if elts else ''):
+            for key, grouper in groupby(entries, keyfn):
                 grouped_entries = [e for e in grouper]
                 sub_entries = [e for e in grouped_entries if len(e[0]) > 1]
                 key_entries = [e for e in grouped_entries if len(e[0]) == 1]
@@ -357,7 +371,7 @@ class RecentChangesMacro(WikiMacroBase):
         entries_per_date = []
         prevdate = None
         for name, version, ts in self.env.db_query(sql, args):
-            if not 'WIKI_VIEW' in formatter.perm('wiki', name, version):
+            if 'WIKI_VIEW' not in formatter.perm('wiki', name, version):
                 continue
             req = formatter.req
             date = user_time(req, format_date, from_utimestamp(ts))
@@ -382,11 +396,11 @@ class RecentChangesMacro(WikiMacroBase):
             for date, entries in entries_per_date)
 
         if group == 'date':
-            out = ((tag.h3(date), tag.ul(entries))
+            out = ((tag.h3(date, class_='section'), tag.ul(entries))
                    for date, entries in items_per_date)
         else:
             out = tag.ul(entries for date, entries in items_per_date)
-        return tag.div(out)
+        return tag.div(out, class_="wikipage")
 
 
 class PageOutlineMacro(WikiMacroBase):
@@ -442,14 +456,14 @@ class PageOutlineMacro(WikiMacroBase):
 
         # TODO: - integrate the rest of the OutlineFormatter directly here
         #       - use formatter.wikidom instead of formatter.source
-        out = StringIO()
+        out = io.StringIO()
         oformatter = OutlineFormatter(self.env, formatter.context)
         oformatter.format(formatter.source, out, max_depth, min_depth,
                           shorten=not inline)
         outline = Markup(out.getvalue())
 
         if title:
-            outline = tag.h4(title) + outline
+            outline = tag.h4(title, class_='section') + outline
         if not inline:
             outline = tag.div(outline, class_='wiki-toc')
         elif not numbered:
@@ -705,7 +719,7 @@ class ImageMacro(WikiMacroBase):
                 attr[key] = desc
         if style:
             attr['style'] = '; '.join('%s:%s' % (k, escape(v))
-                                      for k, v in style.iteritems())
+                                      for k, v in style.items())
         if not WikiSystem(self.env).is_safe_origin(raw_url,
                                                    formatter.context.req):
             attr['crossorigin'] = 'anonymous'  # avoid password prompt
@@ -730,8 +744,6 @@ class MacroListMacro(WikiMacroBase):
     """)
 
     def expand_macro(self, formatter, name, content):
-        from trac.wiki.formatter import system_message
-
         content = content.strip() if content else ''
         name_filter = content.strip('*')
 
@@ -766,6 +778,8 @@ class MacroListMacro(WikiMacroBase):
                             else:
                                 descr = format_to_html(
                                     self.env, formatter.context, descr)
+                        elif descr is None:
+                            continue
                         yield descr, [name for name, descr in pairs]
 
         return tag.div(class_='trac-macrolist')(
@@ -773,7 +787,7 @@ class MacroListMacro(WikiMacroBase):
              len(names) > 1 and tag.p(tag.strong(_("Aliases:")),
                                       [tag.code(' [[', alias, ']]')
                                        for alias in names[1:]]) or None,
-             description or tag.em(_("Sorry, no documentation found")))
+             description or tag.em(_("No documentation found")))
             for description, names in sorted(get_macro_descr(),
                                              key=lambda item: item[1][0]))
 
@@ -820,7 +834,7 @@ class TracIniMacro(WikiMacroBase):
         section_registry = ConfigSection.get_registry(self.compmgr)
         option_registry = Option.get_registry(self.compmgr)
         options = {}
-        for (section, key), option in option_registry.iteritems():
+        for (section, key), option in option_registry.items():
             if section_filter(section) and option_filter(key):
                 options.setdefault(section, {})[key] = option
         if not has_option_filter:
@@ -828,7 +842,7 @@ class TracIniMacro(WikiMacroBase):
                 if section_filter(section):
                     options.setdefault(section, {})
         for section in options:
-            options[section] = sorted(options[section].itervalues(),
+            options[section] = sorted(iter(options[section].values()),
                                       key=lambda option: option.name)
         sections = [(section, section_registry[section].doc
                               if section in section_registry else '')
@@ -882,7 +896,7 @@ class KnownMimeTypesMacro(WikiMacroBase):
             mime_type_filter = args.pop(0).strip().rstrip('*')
 
         mime_types = {}
-        for key, mime_type in mime_map.iteritems():
+        for key, mime_type in mime_map.items():
             if (not mime_type_filter or
                 mime_type.startswith(mime_type_filter)) and key != mime_type:
                 mime_types.setdefault(mime_type, []).append(key)
@@ -899,7 +913,7 @@ class KnownMimeTypesMacro(WikiMacroBase):
                                   style="text-align: left"),
                            tag.td(tag.code(
                                ' '.join(sorted(mime_types[mime_type])))))
-                    for mime_type in sorted(mime_types.keys()))))
+                    for mime_type in sorted(mime_types))))
 
 
 class TracGuideTocMacro(WikiMacroBase):
