@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2018-2020 Edgewall Software
+# Copyright (C) 2018-2021 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -162,8 +162,7 @@ class TracAdminTestCase(TracAdminTestCaseBase):
 
     def execute(self, cmd, *args):
         argstr = ' '.join('"%s"' % a for a in args)
-        return super(TracAdminTestCase, self) \
-               .execute('wiki {} {}'.format(cmd, argstr))
+        return super().execute('wiki {} {}'.format(cmd, argstr))
 
     def assertFileContentMatchesPage(self, names):
         for n in names:
