@@ -1,44 +1,41 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2009 Edgewall Software
+# Copyright (C) 2003-2021 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2004-2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
 
-from __future__ import with_statement
-
 import pkg_resources
 import re
 
-from genshi.builder import tag
-
-from trac.attachment import AttachmentModule
+from trac.attachment import AttachmentModule, Attachment
 from trac.config import IntOption
 from trac.core import *
 from trac.mimeview.api import IContentConverter, Mimeview
-from trac.perm import IPermissionRequestor
+from trac.perm import IPermissionPolicy, IPermissionRequestor
 from trac.resource import *
 from trac.search import ISearchSource, search_to_sql, shorten_result
 from trac.timeline.api import ITimelineEventProvider
 from trac.util import as_int, get_reporter_id
 from trac.util.datefmt import from_utimestamp, to_utimestamp
+from trac.util.html import tag
 from trac.util.text import shorten_line
 from trac.util.translation import _, tag_
 from trac.versioncontrol.diff import get_diff_options, diff_blocks
 from trac.web.api import HTTPBadRequest, IRequestHandler
-from trac.web.chrome import (Chrome, INavigationContributor,
-                             ITemplateProvider, add_ctxtnav, add_link,
+from trac.web.chrome import (Chrome, INavigationContributor, ITemplateProvider,
+                             accesskey, add_ctxtnav, add_link,
                              add_notice, add_script, add_stylesheet,
                              add_warning, prevnext_nav, web_context)
 from trac.wiki.api import IWikiPageManipulator, WikiSystem, validate_page_name
@@ -46,13 +43,6 @@ from trac.wiki.formatter import format_to, OneLinerFormatter
 from trac.wiki.model import WikiPage
 
 
-class InvalidWikiPage(TracError):
-    """Exception raised when a Wiki page fails validation.
-
-    :deprecated: Not used anymore since 0.11
-    """
-
-
 class WikiModule(Component):
 
     implements(IContentConverter, INavigationContributor,
@@ -61,9 +51,18 @@ class WikiModule(Component):
 
     page_manipulators = ExtensionPoint(IWikiPageManipulator)
 
-    max_size = IntOption('wiki', 'max_size', 262144,
-        """Maximum allowed wiki page size in characters. (''since 0.11.2'')""")
+    realm = WikiSystem.realm
 
+    max_size = IntOption('wiki', 'max_size', 262144,
+        """Maximum allowed wiki page size in characters.""")
+
+    default_edit_area_height = IntOption('wiki', 'default_edit_area_height',
+        20,
+        """Default height of the textarea on the wiki edit page.
+        (//Since 1.1.5//)""")
+
+    START_PAGE = property(lambda self: WikiSystem.START_PAGE)
+    TITLE_INDEX_PAGE = property(lambda self: WikiSystem.TITLE_INDEX_PAGE)
     PAGE_TEMPLATES_PREFIX = 'PageTemplates/'
     DEFAULT_PAGE_TEMPLATE = 'DefaultPage'
 
@@ -82,13 +81,14 @@ class WikiModule(Component):
         return 'wiki'
 
     def get_navigation_items(self, req):
-        if 'WIKI_VIEW' in req.perm('wiki', 'WikiStart'):
+        if 'WIKI_VIEW' in req.perm(self.realm, self.START_PAGE):
             yield ('mainnav', 'wiki',
-                   tag.a(_("Wiki"), href=req.href.wiki(), accesskey=1))
-        if 'WIKI_VIEW' in req.perm('wiki', 'TracGuide'):
+                   tag.a(_("Wiki"), href=req.href.wiki(),
+                         accesskey=accesskey(req, 1)))
+        if 'WIKI_VIEW' in req.perm(self.realm, 'TracGuide'):
             yield ('metanav', 'help',
                    tag.a(_("Help/Guide"), href=req.href.wiki('TracGuide'),
-                         accesskey=6))
+                         accesskey=accesskey(req, 6)))
 
     # IPermissionRequestor methods
 
@@ -108,9 +108,11 @@ class WikiModule(Component):
 
     def process_request(self, req):
         action = req.args.get('action', 'view')
-        pagename = req.args.get('page', 'WikiStart')
-        version = req.args.get('version')
-        old_version = req.args.get('old_version')
+        pagename = req.args.get('page', self.START_PAGE)
+        version = None
+        if req.args.get('version'):  # Allow version to be empty
+            version = req.args.getint('version')
+        old_version = req.args.getint('old_version')
 
         if pagename.startswith('/') or pagename.endswith('/') or \
                 '//' in pagename:
@@ -120,20 +122,12 @@ class WikiModule(Component):
             raise TracError(_("Invalid Wiki page name '%(name)s'",
                               name=pagename))
 
-        if version is not None:
-            version_as_int = as_int(version, None)
-            if version_as_int is None:
-                raise ResourceNotFound(
-                    _('No version "%(num)s" for Wiki page "%(name)s"',
-                      num=version, name=pagename))
-            version = version_as_int
-
         page = WikiPage(self.env, pagename)
-        versioned_page = WikiPage(self.env, pagename, version=version)
+        versioned_page = WikiPage(self.env, pagename, version)
 
         req.perm(versioned_page.resource).require('WIKI_VIEW')
 
-        if version and versioned_page.version != int(version):
+        if version and versioned_page.version != version:
             raise ResourceNotFound(
                 _('No version "%(num)s" for Wiki page "%(name)s"',
                   num=version, name=page.name))
@@ -145,7 +139,7 @@ class WikiModule(Component):
                 if 'cancel' in req.args:
                     req.redirect(req.href.wiki(page.name))
 
-                has_collision = int(version) != page.version
+                has_collision = version != page.version
                 for a in ('preview', 'diff', 'merge'):
                     if a in req.args:
                         action = a
@@ -157,6 +151,8 @@ class WikiModule(Component):
                 else:
                     return self._render_editor(req, page, action,
                                                has_collision)
+            elif action == 'edit_comment':
+                self._do_edit_comment(req, versioned_page)
             elif action == 'delete':
                 self._do_delete(req, versioned_page)
             elif action == 'rename':
@@ -176,6 +172,8 @@ class WikiModule(Component):
             return self._render_confirm_rename(req, page)
         elif action == 'edit':
             return self._render_editor(req, page)
+        elif action == 'edit_comment':
+            return self._render_edit_comment(req, versioned_page)
         elif action == 'diff':
             return self._render_diff(req, versioned_page)
         elif action == 'history':
@@ -213,9 +211,10 @@ class WikiModule(Component):
             for field, message in manipulator.validate_wiki_page(req, page):
                 valid = False
                 if field:
-                    add_warning(req, tag_("The Wiki page field '%(field)s'"
+                    add_warning(req, tag_("The Wiki page field %(field)s"
                                           " is invalid: %(message)s",
-                                          field=field, message=message))
+                                          field=tag.strong(field),
+                                          message=message))
                 else:
                     add_warning(req, tag_("Invalid Wiki page: %(message)s",
                                           message=message))
@@ -256,26 +255,44 @@ class WikiModule(Component):
         add_script(req, 'common/js/diff.js')
         return diff_data, changes
 
+    def _do_edit_comment(self, req, page):
+        req.perm(page.resource).require('WIKI_ADMIN')
+
+        redirect_to = req.args.get('redirect_to')
+        version = old_version = None
+        if redirect_to == 'diff':
+            version = page.version
+            old_version = version - 1
+        redirect_href = req.href.wiki(page.name, action=redirect_to,
+                                      version=version, old_version=old_version)
+        if 'cancel' in req.args:
+            req.redirect(redirect_href)
+
+        new_comment = req.args.get('new_comment')
+
+        page.edit_comment(new_comment)
+        add_notice(req, _("The comment of version %(version)s of the page "
+                          "%(name)s has been updated.",
+                          version=page.version, name=page.name))
+        req.redirect(redirect_href)
+
     def _do_delete(self, req, page):
-        if page.readonly:
-            req.perm(page.resource).require('WIKI_ADMIN')
-        else:
-            req.perm(page.resource).require('WIKI_DELETE')
+        req.perm(page.resource).require('WIKI_DELETE')
 
         if 'cancel' in req.args:
             req.redirect(get_resource_url(self.env, page.resource, req.href))
 
-        version = int(req.args.get('version', 0)) or None
-        old_version = int(req.args.get('old_version', 0)) or version
+        version = req.args.getint('version')
+        old_version = req.args.getint('old_version', version)
 
-        with self.env.db_transaction as db:
+        with self.env.db_transaction:
             if version and old_version and version > old_version:
                 # delete from `old_version` exclusive to `version` inclusive:
                 for v in range(old_version, version):
-                    page.delete(v + 1, db)
+                    page.delete(v + 1)
             else:
                 # only delete that `version`, or the whole page if `None`
-                page.delete(version, db)
+                page.delete(version)
 
         if not page.exists:
             add_notice(req, _("The page %(name)s has been deleted.",
@@ -293,10 +310,7 @@ class WikiModule(Component):
             req.redirect(req.href.wiki(page.name))
 
     def _do_rename(self, req, page):
-        if page.readonly:
-            req.perm(page.resource).require('WIKI_ADMIN')
-        else:
-            req.perm(page.resource).require('WIKI_RENAME')
+        req.perm(page.resource).require('WIKI_RENAME')
 
         if 'cancel' in req.args:
             req.redirect(get_resource_url(self.env, page.resource, req.href))
@@ -324,12 +338,12 @@ class WikiModule(Component):
         with self.env.db_transaction as db:
             page.rename(new_name)
             if redirect:
-                redirection = WikiPage(self.env, old_name, db=db)
+                redirection = WikiPage(self.env, old_name)
                 redirection.text = _('See [wiki:"%(name)s"].', name=new_name)
                 author = get_reporter_id(req)
-                comment = u'[wiki:"%s@%d" %s] \u2192 [wiki:"%s"].' % (
+                comment = '[wiki:"%s@%d" %s] \u2192 [wiki:"%s"].' % (
                           new_name, old_version, old_name, new_name)
-                redirection.save(author, comment, req.remote_addr)
+                redirection.save(author, comment)
 
         add_notice(req, _("The page %(old_name)s has been renamed to "
                           "%(new_name)s.", old_name=old_name,
@@ -342,39 +356,36 @@ class WikiModule(Component):
         req.redirect(req.href.wiki(old_name if redirect else new_name))
 
     def _do_save(self, req, page):
-        if page.readonly:
-            req.perm(page.resource).require('WIKI_ADMIN')
-        elif not page.exists:
+        if not page.exists:
             req.perm(page.resource).require('WIKI_CREATE')
         else:
             req.perm(page.resource).require('WIKI_MODIFY')
 
-        if 'WIKI_ADMIN' in req.perm(page.resource):
+        if 'WIKI_CHANGE_READONLY' in req.perm(page.resource):
             # Modify the read-only flag if it has been changed and the user is
             # WIKI_ADMIN
             page.readonly = int('readonly' in req.args)
 
         try:
-            page.save(get_reporter_id(req, 'author'), req.args.get('comment'),
-                      req.remote_addr)
-            add_notice(req, _("Your changes have been saved in version "
-                              "%(version)s.", version=page.version))
-            req.redirect(get_resource_url(self.env, page.resource, req.href,
-                                          version=None))
+            page.save(get_reporter_id(req, 'author'), req.args.get('comment'))
         except TracError:
             add_warning(req, _("Page not modified, showing latest version."))
             return self._render_view(req, page)
 
+        href = req.href.wiki(page.name, action='diff', version=page.version)
+        add_notice(req, tag_("Your changes have been saved in version "
+                             "%(version)s (%(diff)s).", version=page.version,
+                             diff=tag.a(_("diff"), href=href)))
+        req.redirect(get_resource_url(self.env, page.resource, req.href,
+                                      version=None))
+
     def _render_confirm_delete(self, req, page):
-        if page.readonly:
-            req.perm(page.resource).require('WIKI_ADMIN')
-        else:
-            req.perm(page.resource).require('WIKI_DELETE')
+        req.perm(page.resource).require('WIKI_DELETE')
 
         version = None
         if 'delete_version' in req.args:
-            version = int(req.args.get('version', 0))
-        old_version = as_int(req.args.get('old_version'), version)
+            version = req.args.getint('version', 0)
+        old_version = req.args.getint('old_version', version)
 
         what = 'multiple' if version and old_version \
                              and version - old_version > 1 \
@@ -383,34 +394,34 @@ class WikiModule(Component):
         num_versions = 0
         new_date = None
         old_date = None
-        for v, t, author, comment, ipnr in page.get_history():
-            if (v <= version or what == 'page') and new_date is None:
+        for v, t, author, comment in page.get_history():
+            if (what == 'page' or v <= version) and new_date is None:
                 new_date = t
-            if (v <= old_version and what == 'multiple' or
-                num_versions > 1 and what == 'single'):
+            if (what == 'multiple' and v <= old_version or
+                what == 'single' and num_versions > 1):
                 break
             num_versions += 1
             old_date = t
 
         data = self._page_data(req, page, 'delete')
-        data.update({'what': what, 'new_version': None, 'old_version': None,
-                     'num_versions': num_versions, 'new_date': new_date,
-                     'old_date': old_date})
+        attachments = Attachment.select(self.env, self.realm, page.name)
+        data.update({
+            'what': what, 'new_version': None, 'old_version': None,
+            'num_versions': num_versions, 'new_date': new_date,
+            'old_date': old_date, 'attachments': list(attachments),
+        })
         if version is not None:
             data.update({'new_version': version, 'old_version': old_version})
         self._wiki_ctxtnav(req, page)
-        return 'wiki_delete.html', data, None
+        return 'wiki_delete.html', data
 
     def _render_confirm_rename(self, req, page, new_name=None):
-        if page.readonly:
-            req.perm(page.resource).require('WIKI_ADMIN')
-        else:
-            req.perm(page.resource).require('WIKI_RENAME')
+        req.perm(page.resource).require('WIKI_RENAME')
 
         data = self._page_data(req, page, 'rename')
         data['new_name'] = new_name if new_name is not None else page.name
         self._wiki_ctxtnav(req, page)
-        return 'wiki_rename.html', data, None
+        return 'wiki_rename.html', data
 
     def _render_diff(self, req, page):
         if not page.exists:
@@ -418,29 +429,27 @@ class WikiModule(Component):
                               "exist",
                               num=req.args.get('version'), name=page.name))
 
-        old_version = req.args.get('old_version')
+        old_version = req.args.getint('old_version')
         if old_version:
-            old_version = int(old_version)
             if old_version == page.version:
                 old_version = None
             elif old_version > page.version:
                 # FIXME: what about reverse diffs?
                 old_version = page.resource.version
-                page = WikiPage(self.env, page.name, version=old_version)
+                page = WikiPage(self.env, page.name, old_version)
                 req.perm(page.resource).require('WIKI_VIEW')
-        latest_page = WikiPage(self.env, page.name, version=None)
+        latest_page = WikiPage(self.env, page.name)
         req.perm(latest_page.resource).require('WIKI_VIEW')
-        new_version = int(page.version)
+        new_version = page.version
 
-        date = author = comment = ipnr = None
+        date = author = comment = None
         num_changes = 0
         prev_version = next_version = None
-        for version, t, a, c, i in latest_page.get_history():
+        for version, t, a, c in latest_page.get_history():
             if version == new_version:
                 date = t
                 author = a or 'anonymous'
                 comment = c or '--'
-                ipnr = i or ''
             else:
                 if version < new_version:
                     num_changes += 1
@@ -476,30 +485,28 @@ class WikiModule(Component):
 
         data = self._page_data(req, page, 'diff')
         data.update({
-            'change': {'date': date, 'author': author, 'ipnr': ipnr,
-                       'comment': comment},
+            'change': {'date': date, 'author': author, 'comment': comment},
             'new_version': new_version, 'old_version': old_version,
             'latest_version': latest_page.version,
             'num_changes': num_changes,
             'longcol': 'Version', 'shortcol': 'v',
             'changes': changes,
             'diff': diff_data,
+            'can_edit_comment': 'WIKI_ADMIN' in req.perm(page.resource),
         })
         prevnext_nav(req, _("Previous Change"), _("Next Change"),
                      _("Wiki History"))
-        return 'wiki_diff.html', data, None
+        return 'wiki_diff.html', data
 
     def _render_editor(self, req, page, action='edit', has_collision=False):
         if has_collision:
             if action == 'merge':
-                page = WikiPage(self.env, page.name, version=None)
+                page = WikiPage(self.env, page.name)
                 req.perm(page.resource).require('WIKI_VIEW')
             else:
                 action = 'collision'
 
-        if page.readonly:
-            req.perm(page.resource).require('WIKI_ADMIN')
-        elif not page.exists:
+        if not page.exists:
             req.perm(page.resource).require('WIKI_CREATE')
         else:
             req.perm(page.resource).require('WIKI_MODIFY')
@@ -508,25 +515,30 @@ class WikiModule(Component):
         if 'text' in req.args:
             page.text = req.args.get('text')
         elif 'template' in req.args:
-            template = self.PAGE_TEMPLATES_PREFIX + req.args.get('template')
+            template = req.args.get('template')
+            template = template[1:] if template.startswith('/') \
+                                    else self.PAGE_TEMPLATES_PREFIX + template
             template_page = WikiPage(self.env, template)
             if template_page and template_page.exists and \
                     'WIKI_VIEW' in req.perm(template_page.resource):
                 page.text = template_page.text
         elif 'version' in req.args:
-            old_page = WikiPage(self.env, page.name,
-                                version=int(req.args['version']))
-            req.perm(page.resource).require('WIKI_VIEW')
-            page.text = old_page.text
-            comment = _("Reverted to version %(version)s.",
-                        version=req.args['version'])
+            version = None
+            if req.args.get('version'):  # Allow version to be empty
+                version = req.args.as_int('version')
+            if version is not None:
+                old_page = WikiPage(self.env, page.name, version)
+                req.perm(page.resource).require('WIKI_VIEW')
+                page.text = old_page.text
+                comment = _("Reverted to version %(version)s.",
+                            version=version)
         if action in ('preview', 'diff'):
             page.readonly = 'readonly' in req.args
 
         author = get_reporter_id(req, 'author')
-        defaults = {'editrows': '20'}
-        prefs = dict((key, req.session.get('wiki_%s' % key, defaults.get(key)))
-                     for key in ('editrows', 'sidebyside'))
+        defaults = {'editrows': str(self.default_edit_area_height)}
+        prefs = {key: req.session.get('wiki_%s' % key, defaults.get(key))
+                 for key in ('editrows', 'sidebyside')}
 
         if 'from_editor' in req.args:
             sidebyside = req.args.get('sidebyside') or None
@@ -557,7 +569,7 @@ class WikiModule(Component):
             'sidebyside': sidebyside,
             'scroll_bar_pos': req.args.get('scroll_bar_pos', ''),
             'diff': None,
-            'attachments': AttachmentModule(self.env).attachment_data(context),
+            'attachments': AttachmentModule(self.env).attachment_data(context)
         })
         if action in ('diff', 'merge'):
             old_text = original_text.splitlines() if original_text else []
@@ -573,8 +585,15 @@ class WikiModule(Component):
         self._wiki_ctxtnav(req, page)
         Chrome(self.env).add_wiki_toolbars(req)
         Chrome(self.env).add_auto_preview(req)
-        add_script(req, 'common/js/folding.js')
-        return 'wiki_edit.html', data, None
+        add_script(req, 'common/js/wiki.js')
+        return 'wiki_edit.html', data
+
+    def _render_edit_comment(self, req, page):
+        req.perm(page.resource).require('WIKI_ADMIN')
+        data = self._page_data(req, page, 'edit_comment')
+        data.update({'redirect_to': req.args.get('redirect_to', 'history')})
+        self._wiki_ctxtnav(req, page)
+        return 'wiki_edit_comment.html', data
 
     def _render_history(self, req, page):
         """Extract the complete history for a given page.
@@ -588,18 +607,21 @@ class WikiModule(Component):
         data = self._page_data(req, page, 'history')
 
         history = []
-        for version, date, author, comment, ipnr in page.get_history():
+        for version, date, author, comment in page.get_history():
             history.append({
                 'version': version,
                 'date': date,
                 'author': author,
-                'comment': comment,
-                'ipnr': ipnr
+                'comment': comment or ''
             })
-        data.update({'history': history, 'resource': page.resource})
+        data.update({
+            'history': history,
+            'resource': page.resource,
+            'can_edit_comment': 'WIKI_ADMIN' in req.perm(page.resource)
+        })
         add_ctxtnav(req, _("Back to %(wikipage)s", wikipage=page.name),
                     req.href.wiki(page.name))
-        return 'history_view.html', data, None
+        return 'history_view.html', data
 
     def _render_view(self, req, page):
         version = page.resource.version
@@ -609,12 +631,12 @@ class WikiModule(Component):
             for conversion in Mimeview(self.env) \
                               .get_supported_conversions('text/x-trac-wiki'):
                 conversion_href = req.href.wiki(page.name, version=version,
-                                                format=conversion[0])
-                add_link(req, 'alternate', conversion_href, conversion[1],
-                         conversion[3])
+                                                format=conversion.key)
+                add_link(req, 'alternate', conversion_href, conversion.name,
+                         conversion.in_mimetype)
 
         data = self._page_data(req, page)
-        if page.name == 'WikiStart':
+        if page.name == self.START_PAGE:
             data['title'] = ''
 
         ws = WikiSystem(self.env)
@@ -637,18 +659,18 @@ class WikiModule(Component):
             name = name.lower()
             related = [each for each in ws.pages
                        if name in each.lower()
-                          and 'WIKI_VIEW' in req.perm('wiki', each)]
+                          and 'WIKI_VIEW' in req.perm(self.realm, each)]
             related.sort()
             related = [ws._format_link(formatter, 'wiki', '/' + each, each,
                                        False)
                        for each in related]
 
-        latest_page = WikiPage(self.env, page.name, version=None)
+        latest_page = WikiPage(self.env, page.name)
 
         prev_version = next_version = None
         if version:
-            try:
-                version = int(version)
+            version = as_int(version, None)
+            if version is not None:
                 for hist in latest_page.get_history():
                     v = hist[0]
                     if v != version:
@@ -658,13 +680,11 @@ class WikiModule(Component):
                                 break
                         else:
                             next_version = v
-            except ValueError:
-                version = None
 
         prefix = self.PAGE_TEMPLATES_PREFIX
         templates = [template[len(prefix):]
                      for template in ws.get_pages(prefix)
-                     if 'WIKI_VIEW' in req.perm('wiki', template)]
+                     if 'WIKI_VIEW' in req.perm(self.realm, template)]
 
         # -- prev/up/next links
         if prev_version:
@@ -706,19 +726,23 @@ class WikiModule(Component):
             'text': text,
             'latest_version': latest_page.version,
             'attachments': AttachmentModule(self.env).attachment_data(context),
+            'start_page': self.START_PAGE,
             'default_template': self.DEFAULT_PAGE_TEMPLATE,
             'templates': templates,
             'version': version,
             'higher': higher, 'related': related,
             'resourcepath_template': 'wiki_page_path.html',
+            'fullwidth': req.session.get('wiki_fullwidth'),
         })
-        add_script(req, 'common/js/folding.js')
-        return 'wiki_view.html', data, None
+        add_script(req, 'common/js/wiki.js')
+        return 'wiki_view.html', data
 
     def _wiki_ctxtnav(self, req, page):
         """Add the normal wiki ctxtnav entries."""
-        add_ctxtnav(req, _("Start Page"), req.href.wiki('WikiStart'))
-        add_ctxtnav(req, _("Index"), req.href.wiki('TitleIndex'))
+        if 'WIKI_VIEW' in req.perm('wiki', self.START_PAGE):
+            add_ctxtnav(req, _("Start Page"), req.href.wiki(self.START_PAGE))
+        if 'WIKI_VIEW' in req.perm('wiki', self.TITLE_INDEX_PAGE):
+            add_ctxtnav(req, _("Index"), req.href.wiki(self.TITLE_INDEX_PAGE))
         if page.exists:
             add_ctxtnav(req, _("History"), req.href.wiki(page.name,
                                                          action='history'))
@@ -731,7 +755,7 @@ class WikiModule(Component):
 
     def get_timeline_events(self, req, start, stop, filters):
         if 'wiki' in filters:
-            wiki_realm = Resource('wiki')
+            wiki_realm = Resource(self.realm)
             for ts, name, comment, author, version in self.env.db_query("""
                     SELECT time, name, comment, author, version FROM wiki
                     WHERE time>=%s AND time<=%s
@@ -774,12 +798,12 @@ class WikiModule(Component):
             yield ('wiki', _('Wiki'))
 
     def get_search_results(self, req, terms, filters):
-        if not 'wiki' in filters:
+        if 'wiki' not in filters:
             return
         with self.env.db_query as db:
             sql_query, args = search_to_sql(db, ['w1.name', 'w1.author',
                                                  'w1.text'], terms)
-            wiki_realm = Resource('wiki')
+            wiki_realm = Resource(self.realm)
             for name, ts, author, text in db("""
                     SELECT w1.name, w1.time, w1.author, w1.text
                     FROM wiki w1,(SELECT name, max(version) AS ver
@@ -797,3 +821,26 @@ class WikiModule(Component):
         for result in AttachmentModule(self.env).get_search_results(
                 req, wiki_realm, terms):
             yield result
+
+
+class DefaultWikiPolicy(Component):
+    """Default permission policy for the wiki system.
+
+    Wiki pages with the read-only attribute require `WIKI_ADMIN` to delete,
+    modify or rename the page.
+    """
+
+    implements(IPermissionPolicy)
+
+    realm = WikiSystem.realm
+
+    # IPermissionPolicy methods
+
+    def check_permission(self, action, username, resource, perm):
+        if resource and resource.realm == self.realm:
+            if action == 'WIKI_CHANGE_READONLY':
+                return 'WIKI_ADMIN' in perm(resource)
+            if action in ('WIKI_DELETE', 'WIKI_MODIFY', 'WIKI_RENAME'):
+                page = WikiPage(self.env, resource)
+                if page.readonly and 'WIKI_ADMIN' not in perm(resource):
+                    return False
