@@ -1,29 +1,29 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2009 Edgewall Software
+# Copyright (C) 2007-2021 Edgewall Software
 # Copyright (C) 2007 Alec Thomas <alec@swapoff.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Alec Thomas <alec@swapoff.org>
 
+import configparser
 import os
-from ConfigParser import ParsingError
 from fnmatch import fnmatchcase
 from itertools import groupby
 
 from trac.config import ConfigurationError, PathOption, UnicodeConfigParser
-from trac.core import *
+from trac.core import Component, implements
 from trac.perm import IPermissionPolicy, PermissionSystem
 from trac.util import to_list
-from trac.util.text import to_unicode
+from trac.util.text import exception_to_unicode
 
 
 class AuthzPolicy(Component):
@@ -38,15 +38,15 @@ class AuthzPolicy(Component):
     permissions here. Only additional rights or restrictions should be added.
 
     === Installation ===
+    Enabling this policy requires listing it in `trac.ini`::
 
-    Enabling this policy requires listing it in `trac.ini:
-    {{{
-    [trac]
-    permission_policies = AuthzPolicy, DefaultPermissionPolicy
+      {{{
+      [trac]
+      permission_policies = AuthzPolicy, DefaultPermissionPolicy
 
-    [authz_policy]
-    authz_file = conf/authzpolicy.conf
-    }}}
+      [authz_policy]
+      authz_file = conf/authzpolicy.conf
+      }}}
 
     This means that the `AuthzPolicy` permissions will be checked first, and
     only if no rule is found will the `DefaultPermissionPolicy` be used.
@@ -56,31 +56,35 @@ class AuthzPolicy(Component):
     The `authzpolicy.conf` file is a `.ini` style configuration file.
 
      - Each section of the config is a glob pattern used to match against a
-       Trac resource descriptor. These descriptors are in the form:
-       {{{
-       <realm>:<id>@<version>[/<realm>:<id>@<version> ...]
-       }}}
+       Trac resource descriptor. These descriptors are in the form::
+
+         {{{
+         <realm>:<id>@<version>[/<realm>:<id>@<version> ...]
+         }}}
+
        Resources are ordered left to right, from parent to child. If any
        component is inapplicable, `*` is substituted. If the version pattern is
        not specified explicitely, all versions (`@*`) is added implicitly
 
-       Example: Match the WikiStart page
-       {{{
-       [wiki:*]
-       [wiki:WikiStart*]
-       [wiki:WikiStart@*]
-       [wiki:WikiStart]
-       }}}
+       Example: Match the WikiStart page::
 
-       Example: Match the attachment `wiki:WikiStart@117/attachment/FOO.JPG@*`
-       on WikiStart
-       {{{
-       [wiki:*]
-       [wiki:WikiStart*]
-       [wiki:WikiStart@*]
-       [wiki:WikiStart@*/attachment/*]
-       [wiki:WikiStart@117/attachment/FOO.JPG]
-       }}}
+         {{{
+         [wiki:*]
+         [wiki:WikiStart*]
+         [wiki:WikiStart@*]
+         [wiki:WikiStart]
+         }}}
+
+       Example: Match the attachment
+       ``wiki:WikiStart@117/attachment/FOO.JPG@*`` on WikiStart::
+
+         {{{
+         [wiki:*]
+         [wiki:WikiStart*]
+         [wiki:WikiStart@*]
+         [wiki:WikiStart@*/attachment/*]
+         [wiki:WikiStart@117/attachment/FOO.JPG]
+         }}}
 
      - Sections are checked against the current Trac resource '''IN ORDER''' of
        appearance in the configuration file. '''ORDER IS CRITICAL'''.
@@ -91,33 +95,36 @@ class AuthzPolicy(Component):
        denied rather than granted. The username will match any of 'anonymous',
        'authenticated', <username> or '*', using normal Trac permission rules.
 
-    Example configuration:
-    {{{
-    [groups]
-    administrators = athomas
+    Example configuration::
 
-    [*/attachment:*]
-    * = WIKI_VIEW, TICKET_VIEW
+      {{{
+      [groups]
+      administrators = athomas
 
-    [wiki:WikiStart@*]
-    @administrators = WIKI_ADMIN
-    anonymous = WIKI_VIEW
-    * = WIKI_VIEW
+      [*/attachment:*]
+      * = WIKI_VIEW, TICKET_VIEW
 
-    # Deny access to page templates
-    [wiki:PageTemplates/*]
-    * =
+      [wiki:WikiStart@*]
+      @administrators = WIKI_ADMIN
+      anonymous = WIKI_VIEW
+      * = WIKI_VIEW
+
+      # Deny access to page templates
+      [wiki:PageTemplates/*]
+      * =
+
+      # Match everything else
+      [*]
+      @administrators = TRAC_ADMIN
+      anonymous = BROWSER_VIEW, CHANGESET_VIEW, FILE_VIEW, LOG_VIEW,
+          MILESTONE_VIEW, POLL_VIEW, REPORT_SQL_VIEW, REPORT_VIEW,
+          ROADMAP_VIEW, SEARCH_VIEW, TICKET_CREATE, TICKET_MODIFY,
+          TICKET_VIEW, TIMELINE_VIEW,
+          WIKI_CREATE, WIKI_MODIFY, WIKI_VIEW
+      # Give authenticated users some extra permissions
+      authenticated = REPO_SEARCH, XML_RPC
+      }}}
 
-    # Match everything else
-    [*]
-    @administrators = TRAC_ADMIN
-    anonymous = BROWSER_VIEW, CHANGESET_VIEW, FILE_VIEW, LOG_VIEW,
-        MILESTONE_VIEW, POLL_VIEW, REPORT_SQL_VIEW, REPORT_VIEW, ROADMAP_VIEW,
-        SEARCH_VIEW, TICKET_CREATE, TICKET_MODIFY, TICKET_VIEW, TIMELINE_VIEW,
-        WIKI_CREATE, WIKI_MODIFY, WIKI_VIEW
-    # Give authenticated users some extra permissions
-    authenticated = REPO_SEARCH, XML_RPC
-    }}}
     """
     implements(IPermissionPolicy)
 
@@ -126,21 +133,9 @@ class AuthzPolicy(Component):
                             "Non-absolute paths are relative to the "
                             "Environment `conf` directory.")
 
-    authz = None
-    authz_mtime = None
-
     def __init__(self):
-        if not self.authz_file:
-            self.log.error("The `[authz_policy] authz_file` configuration "
-                           "option in trac.ini is empty or not defined.")
-            raise ConfigurationError()
-
-        try:
-            os.stat(self.authz_file)
-        except OSError as e:
-            self.log.error("Error parsing authz permission policy file: %s",
-                           to_unicode(e))
-            raise ConfigurationError()
+        self.authz = None
+        self.authz_mtime = None
         self.groups_by_user = {}
 
     # IPermissionPolicy methods
@@ -174,12 +169,23 @@ class AuthzPolicy(Component):
         self.log.debug("Parsing authz security policy %s",
                        self.authz_file)
 
-        self.authz = UnicodeConfigParser()
+        if not self.authz_file:
+            self.log.error("The `[authz_policy] authz_file` configuration "
+                           "option in trac.ini is empty or not defined.")
+            raise ConfigurationError()
+        try:
+            authz_mtime = os.path.getmtime(self.authz_file)
+        except OSError as e:
+            self.log.error("Error parsing authz permission policy file: %s",
+                           exception_to_unicode(e))
+            raise ConfigurationError()
+
+        self.authz = UnicodeConfigParser(ignorecase_option=False)
         try:
             self.authz.read(self.authz_file)
-        except ParsingError as e:
+        except configparser.ParsingError as e:
             self.log.error("Error parsing authz permission policy file: %s",
-                           to_unicode(e))
+                           exception_to_unicode(e))
             raise ConfigurationError()
         groups = {}
         if self.authz.has_section('groups'):
@@ -195,10 +201,23 @@ class AuthzPolicy(Component):
                 else:
                     self.groups_by_user.setdefault(item, set()).add(group)
 
-        for group, users in groups.iteritems():
+        for group, users in groups.items():
             add_items('@' + group, users)
 
-        self.authz_mtime = os.path.getmtime(self.authz_file)
+        all_actions = set(PermissionSystem(self.env).get_actions())
+        authz_basename = os.path.basename(self.authz_file)
+        for section in self.authz.sections():
+            if section == 'groups':
+                continue
+            for user, actions in self.authz.items(section):
+                for action in to_list(actions):
+                    if action.startswith('!'):
+                        action = action[1:]
+                    if action not in all_actions:
+                        self.log.warning("The action %s in the [%s] section "
+                                         "of %s is not a valid action.",
+                                         action, section, authz_basename)
+        self.authz_mtime = authz_mtime
 
     def normalise_resource(self, resource):
         def to_descriptor(resource):
@@ -230,7 +249,7 @@ class AuthzPolicy(Component):
         # TODO: Handle permission negation in sections. eg. "if in this
         # ticket, remove TICKET_MODIFY"
         if username and username != 'anonymous':
-            valid_users = ['*', 'authenticated', username]
+            valid_users = ['*', 'authenticated', 'anonymous', username]
         else:
             valid_users = ['*', 'anonymous']
         for resource_section in [a for a in self.authz.sections()
@@ -246,7 +265,7 @@ class AuthzPolicy(Component):
                             who in self.groups_by_user.get(username, []):
                         self.log.debug("%s matched section %s for user %s",
                                        resource_key, resource_glob, username)
-                        if isinstance(permissions, basestring):
+                        if isinstance(permissions, str):
                             return [permissions]
                         else:
                             return permissions
